@@ -1,5 +1,5 @@
 /*
- *  cddb.c  Copyright 1999-2000 Hvard Kvlen <havardk@sol.no>
+ *  cddb.c  Copyright 1999-2001 Hvard Kvlen <havardk@xmms.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -19,19 +19,23 @@
 
 
 #include "cddb.h"
+#include "http.h"
 #include "cdaudio.h"
 #include "cdinfo.h"
-#include "http.h"
 #include <libxmms/util.h>
 #include <gtk/gtk.h>
 #include <stdarg.h>
 #include <dirent.h>
+#include <pthread.h>
 
 
 static guint32 cached_id = 0;
 static GtkWidget *server_dialog, *server_clist;
 static GtkWidget *debug_window, *debug_clist;
 static GList *debug_messages = NULL;
+static GList *temp_messages = NULL;
+static pthread_mutex_t list_mutex = PTHREAD_MUTEX_INITIALIZER;
+static guint cddb_timeout_id;
 
 void configure_set_cddb_server(gchar *server);
 
@@ -40,14 +44,14 @@ static void cddb_log(gchar *str, ...)
 	static GList *end_ptr = NULL;
 	static gint message_num = 0;
 	va_list args;
-	gchar *temp;
+	char *text;
 
 	va_start(args, str);
-	temp = g_strdup_vprintf(str, args);
+	text = g_strdup_vprintf(str, args);
 	va_end(args);
 
 	message_num++;
-	debug_messages = g_list_prepend(debug_messages, temp);
+	debug_messages = g_list_prepend(debug_messages, text);
 	if (!end_ptr)
 		end_ptr = debug_messages;
 	if (message_num > CDDB_LOG_MAX)
@@ -61,16 +65,12 @@ static void cddb_log(gchar *str, ...)
 		end_ptr = temp;
 		message_num--;
 	}
-/*  	if (debug_window) */
-/*  	{ */
-/*  		GDK_THREADS_ENTER(); */
-/*  		gtk_clist_freeze(GTK_CLIST(debug_clist)); */
-/*  		gtk_clist_append(GTK_CLIST(debug_clist), &temp); */
-/*  		gtk_clist_columns_autosize(GTK_CLIST(debug_clist)); */
-/*  		gtk_clist_thaw(GTK_CLIST(debug_clist)); */
-/*  		gtk_clist_moveto(GTK_CLIST(debug_clist), GTK_CLIST(debug_clist)->rows - 1, -1, 0.5, 0); */
-/*  		GDK_THREADS_LEAVE(); */
-/*  	} */
+	if (debug_window)
+	{
+		pthread_mutex_lock(&list_mutex);
+		temp_messages = g_list_append(temp_messages, g_strdup(text));
+		pthread_mutex_unlock(&list_mutex);
+	}
 }
 
 static gint cddb_sum(gint in)
@@ -100,31 +100,50 @@ guint32 cdda_cddb_compute_discid(cdda_disc_toc_t *info)
 	return ((high % 0xff) << 24 | low << 8 | (info->last_track - info->first_track + 1));
 }
 
-static gchar * cddb_generate_offset_string(cdda_disc_toc_t *info)
+static char * cddb_generate_offset_string(cdda_disc_toc_t *info)
 {
-	gchar buffer[6*99];
-	gint i;
+	char *buffer;
+	int i;
+
+	buffer = g_malloc(info->last_track * 7 + 1);
 
 	sprintf(buffer, "%d", LBA(info->track[info->first_track]));
 
 	for (i = info->first_track + 1; i <= info->last_track; i++)
 		sprintf(buffer, "%s+%d", buffer, LBA(info->track[i]));
 
-	return g_strdup(buffer);
+	return buffer;
 }
 
 static gchar * cddb_generate_hello_string(void)
 {
-	gchar hostn[CDDB_HOSTNAME_LEN];
-	static gchar *buffer = NULL;
+	static char *buffer;
 
 	if (buffer == NULL)
 	{
-		if (gethostname(hostn, CDDB_HOSTNAME_LEN) < 0)
-			strcpy(hostn, "localhost"); /* Failed, supply a dummy name */
+		char *env, *client = NULL, *version = NULL, **strs = NULL;
 
-		buffer = g_strdup_printf("&hello=%s+%s+%s+%s",
-					 g_get_user_name(), hostn, PACKAGE, VERSION);
+		env = getenv("XMMS_CDDB_CLIENT_NAME");
+		if (env)
+		{
+			strs = g_strsplit(env, " ", 2);
+			if (strs && strs[0] && strs[1])
+			{
+				client = strs[0];
+				version = strs[1];
+			}
+		}
+
+		if (!client || !version)
+		{
+			client = PACKAGE;
+			version = VERSION;
+		}
+		
+		buffer = g_strdup_printf("&hello=nobody+localhost+%s+%s",
+					 client, version);
+		if (strs)
+			g_strfreev(strs);
 	}
 	return buffer;
 }
@@ -211,10 +230,8 @@ static gboolean cddb_query(gchar *server, cdda_disc_toc_t *info, cddb_disc_heade
 
 static gint cddb_check_protocol_level(gchar *server)
 {
-	gint level = 0;
-	gchar *str;
-	gchar buffer[256];
-	gint sock;
+	int level = 0, sock, n;
+	char *str, buffer[256];
 
 	if((sock = cddb_http_open_connection(server, 80)) == 0)
 		return 0;
@@ -226,10 +243,16 @@ static gint cddb_check_protocol_level(gchar *server)
 	write(sock, str, strlen(str));
 	g_free(str);
 	
-	if (http_read_first_line(sock, buffer, 256) < 0 || atoi(buffer) != 210)
+	if ((n = http_read_first_line(sock, buffer, 256)) < 0 ||
+	    atoi(buffer) != 210)
 	{
+		if (n > 0)
+			cddb_log("Getting cddb protocol level failed: %s",
+				 buffer);
+		else
+			cddb_log("Getting cddb protocol level failed.");
+			
 		http_close_connection(sock);
-		cddb_log("Getting cddb protocol level: Failed");
 		return 0;
 	}
 
@@ -336,7 +359,8 @@ static gboolean cddb_read(gchar *server, cddb_disc_header_t *cddb_info, cdinfo_t
 				}
 				if (oldnum >= 0)
 					cdda_cdinfo_track_set(cdinfo, oldnum + 1, NULL, g_strdup(buffer2));
-				bufs = num = oldnum = 0;
+				bufs = 0;
+				oldnum = -1;
 				command++;
 			case 4:
 				if (!strncmp(buffer, "EXTD", 4))
@@ -363,23 +387,18 @@ static gboolean cddb_read(gchar *server, cddb_disc_header_t *cddb_info, cdinfo_t
 		
 	} while (http_read_line(sock, buffer, 256) >= 0);
 	
+	if (oldnum >= 0)
+		cdda_cdinfo_track_set(cdinfo, oldnum + 1, NULL, g_strdup(buffer2));
+
 	http_close_connection(sock);
 	return TRUE;
 }
 
 static gint cddb_get_protocol_level(void)
 {
-	ConfigFile *cfgfile;
-
 	if (cdda_cfg.cddb_protocol_level < 1)
-	{
-		if ((cdda_cfg.cddb_protocol_level = cddb_check_protocol_level(cdda_cfg.cddb_server)) == 0)
-			return 0;
-		cfgfile = xmms_cfg_open_default_file();
-		xmms_cfg_write_int(cfgfile, "CDDA", "cddb_protocol_level", cdda_cfg.cddb_protocol_level);
-		xmms_cfg_write_default_file(cfgfile);
-		xmms_cfg_free(cfgfile);
-	}
+		cdda_cfg.cddb_protocol_level =
+			cddb_check_protocol_level(cdda_cfg.cddb_server);
 	
 	return cdda_cfg.cddb_protocol_level;
 }
@@ -526,6 +545,8 @@ int cddb_read_file(char *file, cddb_disc_header_t *cddb_info, cdinfo_t *cdinfo)
 
         realstr++;
         len = strlen(realstr);
+	if (realstr[len-1] == '\n')
+	    realstr[--len] = '\0';	/* remove newline */
 
         switch (command) {
             case 1:
@@ -572,7 +593,8 @@ int cddb_read_file(char *file, cddb_disc_header_t *cddb_info, cdinfo_t *cdinfo)
                 }
                 if (oldnum >= 0)
                     cdda_cdinfo_track_set(cdinfo, oldnum + 1, NULL, g_strdup(buffer2));
-                bufs = num = oldnum = 0;
+		bufs = 0;
+		oldnum = -1;
                 command++;
             case 4:
                 if (!strncmp(buffer, "EXTD", 4))
@@ -597,7 +619,10 @@ int cddb_read_file(char *file, cddb_disc_header_t *cddb_info, cdinfo_t *cdinfo)
                 break;
         }
 
-    };
+    }
+
+    if (oldnum >= 0)
+	cdda_cdinfo_track_set(cdinfo, oldnum + 1, NULL, g_strdup(buffer2));
 
     fclose(fd);
     return(1);
@@ -663,12 +688,14 @@ static gchar* cddb_position_string(gchar * input)
 
 static void cddb_server_dialog_ok_cb(GtkWidget *w, gpointer data)
 {
-	gchar *text;
+	char *text;
+	int pos;
 	GtkEntry *entry = GTK_ENTRY(data);
 
 	if (!GTK_CLIST(server_clist)->selection)
 		return;
-	gtk_clist_get_text(GTK_CLIST(server_clist), (gint) GTK_CLIST(server_clist)->selection->data, 0, &text);
+	pos = GPOINTER_TO_INT(GTK_CLIST(server_clist)->selection->data);
+	gtk_clist_get_text(GTK_CLIST(server_clist), pos, 0, &text);
 	cdda_cddb_set_server(text);
 	gtk_entry_set_text(entry, text);
 	gtk_widget_destroy(server_dialog);
@@ -688,8 +715,6 @@ void cdda_cddb_show_server_dialog(GtkWidget *w, gpointer data)
 	GList *servers;
 	gchar *server;
 	gint level;
-	gchar *row[4];
-	gint i;
 
 	if (server_dialog)
 		return;
@@ -754,10 +779,13 @@ void cdda_cddb_show_server_dialog(GtkWidget *w, gpointer data)
 
 	while (servers)
 	{
-		row[0] = g_strdup(((gchar**)servers->data)[0]);
-		row[1] = cddb_position_string(((gchar**)servers->data)[4]);
-		row[2] = cddb_position_string(((gchar**)servers->data)[5]);
-		row[3] = g_strdup(((gchar**)servers->data)[6]);
+		char *row[4];
+		int i;
+		
+		row[0] = g_strdup(((char**) servers->data)[0]);
+		row[1] = cddb_position_string(((char**) servers->data)[4]);
+		row[2] = cddb_position_string(((char**) servers->data)[5]);
+		row[3] = g_strdup(((char**) servers->data)[6]);
 		gtk_clist_append(GTK_CLIST(server_clist), row);
 		for (i = 0; i < 4; i++)
 			g_free(row[i]);
@@ -769,6 +797,39 @@ void cdda_cddb_show_server_dialog(GtkWidget *w, gpointer data)
 	gtk_widget_show_all(server_dialog);
 }
 
+static gboolean cddb_update_log_window(gpointer data)
+{
+	if (!debug_window)
+	{
+		cddb_timeout_id = 0;
+		return FALSE;
+	}
+
+	pthread_mutex_lock(&list_mutex);
+	if (temp_messages != NULL)
+	{
+		GList *temp;
+		GDK_THREADS_ENTER();
+		gtk_clist_freeze(GTK_CLIST(debug_clist));
+		for (temp = temp_messages; temp; temp = temp->next)
+		{
+			char *text = temp->data;
+			gtk_clist_append(GTK_CLIST(debug_clist), &text);
+			g_free(text);
+		}
+		gtk_clist_columns_autosize(GTK_CLIST(debug_clist));
+		gtk_clist_thaw(GTK_CLIST(debug_clist));
+		gtk_clist_moveto(GTK_CLIST(debug_clist),
+				 GTK_CLIST(debug_clist)->rows - 1, -1, 0.5, 0);
+		GDK_THREADS_LEAVE();
+		g_list_free(temp_messages);
+		temp_messages = NULL;
+	}
+	pthread_mutex_unlock(&list_mutex);
+	return TRUE;
+}
+
+
 void cdda_cddb_show_network_window(GtkWidget *w, gpointer data)
 {
 	GtkWidget *vbox, *bbox, *close, *scroll_win;
@@ -817,5 +878,14 @@ void cdda_cddb_show_network_window(GtkWidget *w, gpointer data)
 	gtk_clist_columns_autosize(GTK_CLIST(debug_clist));
 	gtk_clist_set_button_actions(GTK_CLIST(debug_clist), 0, GTK_BUTTON_IGNORED);
 	gtk_clist_moveto(GTK_CLIST(debug_clist), GTK_CLIST(debug_clist)->rows - 1, -1, 0, 0);
+
+	cddb_timeout_id = gtk_timeout_add(500, cddb_update_log_window, NULL);
 	gtk_widget_show_all(debug_window);
 }
+
+void cddb_quit(void)
+{
+	if (cddb_timeout_id)
+		gtk_timeout_remove(cddb_timeout_id);
+	cddb_timeout_id = 0;
+}
