@@ -25,6 +25,7 @@
 #include "mikmod-plugin.h"
 #include "libxmms/configfile.h"
 #include "libxmms/util.h"
+#include "libxmms/titlestring.h"
 #include <gtk/gtk.h>
 
 #include "mikmod.xpm"
@@ -218,9 +219,11 @@ static void init(void)
 
 static int is_our_file(char *filename)
 {
-	char *ext;
+	char *ext, *tmp;
 
+	tmp = strrchr(filename, '/');
 	ext = strrchr(filename, '.');
+	
 	if (ext)
 	{
 		if (!strcasecmp(ext, ".669"))
@@ -255,18 +258,51 @@ static int is_our_file(char *filename)
 			return 1;
 		if (!strcasecmp(ext, ".stx"))
 			return 1;
-		
 	}
-	if (!strcasecmp(filename, "mod."))
+
+	if (tmp)
+	{
+		if(!strncasecmp("/mod.", tmp, 5))
 			return 1;
+	}
+
 	return 0;
 }
 
+static gchar *get_title(gchar *filename)
+{
+	TitleInput *input;
+	gchar *temp, *ext, *title;
+
+	if((temp = Player_LoadTitle(filename)) != NULL)
+		title = g_strdup(temp);
+	else
+	{
+		XMMS_NEW_TITLEINPUT(input);
+
+		temp = g_strdup(filename);
+		ext = strrchr(temp, '.');
+		if (ext)
+			*ext = '\0';
+		input->file_name = g_basename(temp);
+		input->file_ext = ext ? ext+1 : NULL;
+		input->file_path = temp;
+
+		title = xmms_get_titlestring(xmms_get_gentitle_format(),
+					     input);
+		if ( title == NULL )
+			title = g_strdup(input->file_name);
+
+		g_free(temp);
+		g_free(input);
+	}
+
+	return title;
+}
+
 static void get_song_info(char *filename, char **title, int *length)
 {
-	gchar *temp;
-	if((temp = Player_LoadTitle(filename)) != NULL)
-		(*title) = g_strdup(temp);
+	(*title) = get_title(filename);
 	(*length) = -1;
 }
 
