@@ -1,6 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999,2000  Hvard Kvlen
+ *  Copyright (C) 1998-2002  Peter Alm, Mikael Alm, Olle Hallnas,
+ *                           Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1999-2002  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,6 +21,9 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <stdarg.h>
+#include <string.h>
+#include <errno.h>
+#include <libxmms/xentry.h>
 #include "mpg123.h"
 
 static GtkWidget *window = NULL;
@@ -81,6 +85,7 @@ static void save_cb(GtkWidget * w, gpointer data)
 {
 	gint fd;
 	struct id3v1tag_t tag;
+	char *msg = NULL;
 
 	if (!strncasecmp(current_filename, "http://", 7))
 		return;
@@ -113,16 +118,26 @@ static void save_cb(GtkWidget * w, gpointer data)
 		else
 			get_entry_tag(GTK_ENTRY(comment_entry), tag.u.v1_0.comment, 30);
 		tag.genre = find_genre_id(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(genre_combo)->entry)));
-		if (write(fd, &tag, sizeof (struct id3v1tag_t)) != sizeof (struct id3v1tag_t))
-			xmms_show_message(_("File Info"), _("Couldn't write tag!"),
-					  _("Ok"), FALSE, NULL, NULL);
-
+		if (write(fd, &tag, sizeof (tag)) != sizeof (tag))
+			msg = g_strdup_printf(_("%s\nUnable to write to file: %s"),
+					      _("Couldn't write tag!"),
+					      strerror(errno));
 		close(fd);
 	}
 	else
-		xmms_show_message(_("File Info"), _("Couldn't write tag!"), _("Ok"),
-				  FALSE, NULL, NULL);
-	gtk_widget_destroy(window);
+		msg = g_strdup_printf(_("%s\nUnable to open file: %s"),
+				      _("Couldn't write tag!"),
+				      strerror(errno));
+	if (msg)
+	{
+		GtkWidget *mwin = xmms_show_message(_("File Info"), msg, _("Ok"),
+						    FALSE, NULL, NULL);
+		gtk_window_set_transient_for(GTK_WINDOW(mwin),
+					     GTK_WINDOW(window));
+		g_free(msg);
+	}
+	else
+		gtk_widget_destroy(window);
 }
 
 static void label_set_text(GtkWidget * label, char *str, ...)
@@ -141,6 +156,7 @@ static void remove_id3_cb(GtkWidget * w, gpointer data)
 {
 	gint fd, len;
 	struct id3v1tag_t tag;
+	char *msg = NULL;
 
 	if (!strncasecmp(current_filename, "http://", 7))
 		return;
@@ -153,19 +169,29 @@ static void remove_id3_cb(GtkWidget * w, gpointer data)
 		if (!strncmp(tag.tag, "TAG", 3))
 		{
 			if (ftruncate(fd, len))
-				xmms_show_message(_("File Info"),
-						  _("Couldn't remove tag!"), _("Ok"),
-						  FALSE, NULL, NULL);
+				msg = g_strdup_printf(_("%s\n"
+							"Unable to truncate file: %s"),
+						      _("Couldn't remove tag!"),
+						      strerror(errno));
 		}
 		else
-			xmms_show_message(_("File Info"), _("No tag to remove!"), _("Ok"),
-					  FALSE, NULL, NULL);
+			msg = strdup(_("No tag to remove!"));
 		close(fd);
 	}
 	else
-		xmms_show_message(_("File Info"), _("Couldn't remove tag!"),
-				  _("Ok"), FALSE, NULL, NULL);
-	gtk_widget_destroy(window);
+		msg = g_strdup_printf(_("%s\nUnable to open file: %s"),
+				      _("Couldn't remove tag!"),
+				      strerror(errno));
+	if (msg)
+	{
+		GtkWidget *mwin = xmms_show_message(_("File Info"), msg, _("Ok"),
+						    FALSE, NULL, NULL);
+		gtk_window_set_transient_for(GTK_WINDOW(mwin),
+					     GTK_WINDOW(window));
+		g_free(msg);
+	}
+	else
+		gtk_widget_destroy(window);
 }
 
 static void set_mpeg_level_label(gboolean mpeg25, gint lsf, gint layer)
@@ -235,7 +261,7 @@ void mpg123_file_info_box(char *filename)
 
 		label = gtk_label_new(_("Filename:"));
 		gtk_box_pack_start(GTK_BOX(filename_hbox), label, FALSE, TRUE, 0);
-		filename_entry = gtk_entry_new();
+		filename_entry = xmms_entry_new();
 		gtk_editable_set_editable(GTK_EDITABLE(filename_entry), FALSE);
 		gtk_box_pack_start(GTK_BOX(filename_hbox), filename_entry, TRUE, TRUE, 0);
 		
