@@ -30,6 +30,12 @@ static real tfcos12[3];
 static real cos9[3], cos18[3];
 #endif
 
+#ifdef USE_SIMD
+# define DCT36 (fr->dct36)
+#else
+# define DCT36 mpg123_dct36
+#endif
+
 struct bandInfoStruct
 {
 	int longIdx[23];
@@ -118,7 +124,6 @@ void mpg123_init_layer3(int down_sample_sblimit)
 
 	for (i = -256; i < 118 + 4; i++)
 		gainpow2[i + 256] = pow((double) 2.0, -0.25 * (double) (i + 210));
-
 	for (i = 0; i < 8207; i++)
 		ispow[i] = pow((double) i, (double) 4.0 / 3.0);
 
@@ -1366,192 +1371,6 @@ static void III_antialias(real xr[SBLIMIT][SSLIMIT], struct gr_info_s *gr_info)
 /*                                                                  */
 /*------------------------------------------------------------------*/
 
-#ifdef USE_3DNOW
-void dct36(real *inbuf,real *o1,real *o2,real *wintab,real *tsbuf)
-#else
-static void dct36(real * inbuf, real * o1, real * o2, real * wintab, real * tsbuf)
-#endif
-{
-#ifdef NEW_DCT9
-	real tmp[18];
-#endif
-
-	{
-		register real *in = inbuf;
-
-		in[17] += in[16];
-		in[16] += in[15];
-		in[15] += in[14];
-		in[14] += in[13];
-		in[13] += in[12];
-		in[12] += in[11];
-		in[11] += in[10];
-		in[10] += in[9];
-		in[9] += in[8];
-		in[8] += in[7];
-		in[7] += in[6];
-		in[6] += in[5];
-		in[5] += in[4];
-		in[4] += in[3];
-		in[3] += in[2];
-		in[2] += in[1];
-		in[1] += in[0];
-
-		in[17] += in[15];
-		in[15] += in[13];
-		in[13] += in[11];
-		in[11] += in[9];
-		in[9] += in[7];
-		in[7] += in[5];
-		in[5] += in[3];
-		in[3] += in[1];
-
-
-#ifdef NEW_DCT9
-#if 1
-		{
-			real t3;
-			{
-				real t0, t1, t2;
-
-				t0 = COS6_2 * (in[8] + in[16] - in[4]);
-				t1 = COS6_2 * in[12];
-
-				t3 = in[0];
-				t2 = t3 - t1 - t1;
-				tmp[1] = tmp[7] = t2 - t0;
-				tmp[4] = t2 + t0 + t0;
-				t3 += t1;
-
-				t2 = COS6_1 * (in[10] + in[14] - in[2]);
-				tmp[1] -= t2;
-				tmp[7] += t2;
-			}
-			{
-				real t0, t1, t2;
-
-				t0 = cos9[0] * (in[4] + in[8]);
-				t1 = cos9[1] * (in[8] - in[16]);
-				t2 = cos9[2] * (in[4] + in[16]);
-
-				tmp[2] = tmp[6] = t3 - t0 - t2;
-				tmp[0] = tmp[8] = t3 + t0 + t1;
-				tmp[3] = tmp[5] = t3 - t1 + t2;
-			}
-		}
-		{
-			real t1, t2, t3;
-
-			t1 = cos18[0] * (in[2] + in[10]);
-			t2 = cos18[1] * (in[10] - in[14]);
-			t3 = COS6_1 * in[6];
-
-			{
-				real t0 = t1 + t2 + t3;
-				tmp[0] += t0;
-				tmp[8] -= t0;
-			}
-
-			t2 -= t3;
-			t1 -= t3;
-
-			t3 = cos18[2] * (in[2] + in[14]);
-
-			t1 += t3;
-			tmp[3] += t1;
-			tmp[5] -= t1;
-
-			t2 -= t3;
-			tmp[2] += t2;
-			tmp[6] -= t2;
-		}
-
-#else
-		{
-			real t0, t1, t2, t3, t4, t5, t6, t7;
-
-			t1 = COS6_2 * in[12];
-			t2 = COS6_2 * (in[8] + in[16] - in[4]);
-
-			t3 = in[0] + t1;
-			t4 = in[0] - t1 - t1;
-			t5 = t4 - t2;
-			tmp[4] = t4 + t2 + t2;
-
-			t0 = cos9[0] * (in[4] + in[8]);
-			t1 = cos9[1] * (in[8] - in[16]);
-
-			t2 = cos9[2] * (in[4] + in[16]);
-
-			t6 = t3 - t0 - t2;
-			t0 += t3 + t1;
-			t3 += t2 - t1;
-
-			t2 = cos18[0] * (in[2] + in[10]);
-			t4 = cos18[1] * (in[10] - in[14]);
-			t7 = COS6_1 * in[6];
-
-			t1 = t2 + t4 + t7;
-			tmp[0] = t0 + t1;
-			tmp[8] = t0 - t1;
-			t1 = cos18[2] * (in[2] + in[14]);
-			t2 += t1 - t7;
-
-			tmp[3] = t3 + t2;
-			t0 = COS6_1 * (in[10] + in[14] - in[2]);
-			tmp[5] = t3 - t2;
-
-			t4 -= t1 + t7;
-
-			tmp[1] = t5 - t0;
-			tmp[7] = t5 + t0;
-			tmp[2] = t6 + t4;
-			tmp[6] = t6 - t4;
-		}
-#endif
-
-		{
-			real t0, t1, t2, t3, t4, t5, t6, t7;
-
-			t1 = COS6_2 * in[13];
-			t2 = COS6_2 * (in[9] + in[17] - in[5]);
-
-			t3 = in[1] + t1;
-			t4 = in[1] - t1 - t1;
-			t5 = t4 - t2;
-
-			t0 = cos9[0] * (in[5] + in[9]);
-			t1 = cos9[1] * (in[9] - in[17]);
-
-			tmp[13] = (t4 + t2 + t2) * tfcos36[17 - 13];
-			t2 = cos9[2] * (in[5] + in[17]);
-
-			t6 = t3 - t0 - t2;
-			t0 += t3 + t1;
-			t3 += t2 - t1;
-
-			t2 = cos18[0] * (in[3] + in[11]);
-			t4 = cos18[1] * (in[11] - in[15]);
-			t7 = COS6_1 * in[7];
-
-			t1 = t2 + t4 + t7;
-			tmp[17] = (t0 + t1) * tfcos36[17 - 17];
-			tmp[9] = (t0 - t1) * tfcos36[17 - 9];
-			t1 = cos18[2] * (in[3] + in[15]);
-			t2 += t1 - t7;
-
-			tmp[14] = (t3 + t2) * tfcos36[17 - 14];
-			t0 = COS6_1 * (in[11] + in[15] - in[3]);
-			tmp[12] = (t3 - t2) * tfcos36[17 - 12];
-
-			t4 -= t1 + t7;
-
-			tmp[16] = (t5 - t0) * tfcos36[17 - 16];
-			tmp[10] = (t5 + t0) * tfcos36[17 - 10];
-			tmp[15] = (t6 + t4) * tfcos36[17 - 15];
-			tmp[11] = (t6 - t4) * tfcos36[17 - 11];
-		}
-
 #define MACRO(v)						\
 do {								\
 	real tmpval;						\
@@ -1564,120 +1383,156 @@ do {								\
 	ts[SBLIMIT*(9+(v))] = out1[9+(v)] + tmpval * w[9+(v)];	\
 } while (0)
 
-		{
-			register real *out2 = o2;
-			register real *w = wintab;
-			register real *out1 = o1;
-			register real *ts = tsbuf;
-
-			MACRO(0);
-			MACRO(1);
-			MACRO(2);
-			MACRO(3);
-			MACRO(4);
-			MACRO(5);
-			MACRO(6);
-			MACRO(7);
-			MACRO(8);
-		}
-
-#else
-
-		{
-
-#define MACRO0(v)						\
-do {								\
-	real tmp;						\
-	out2[9+(v)] = (tmp = sum0 + sum1) * w[27+(v)];		\
-	out2[8-(v)] = tmp * w[26-(v)];				\
-	sum0 -= sum1;						\
-	ts[SBLIMIT*(8-(v))] = out1[8-(v)] + sum0 * w[8-(v)];	\
-	ts[SBLIMIT*(9+(v))] = out1[9+(v)] + sum0 * w[9+(v)];	\
-} while (0)
-
-#define MACRO1(v)				\
-do {						\
-	real sum0,sum1;				\
-	sum0 = tmp1a + tmp2a;			\
-	sum1 = (tmp1b + tmp2b) * tfcos36[(v)];	\
-	MACRO0(v);				\
-} while (0)
-
-#define MACRO2(v)				\
-do {						\
-	real sum0, sum1;			\
-	sum0 = tmp2a - tmp1a;			\
-	sum1 = (tmp2b - tmp1b) * tfcos36[(v)];	\
-	MACRO0(v);				\
-} while (0)
-
-			register const real *c = COS9;
-			register real *out2 = o2;
-			register real *w = wintab;
-			register real *out1 = o1;
-			register real *ts = tsbuf;
-
-			real ta33, ta66, tb33, tb66;
-
-			ta33 = in[2 * 3 + 0] * c[3];
-			ta66 = in[2 * 6 + 0] * c[6] + in[2 * 0 + 0];
-			tb33 = in[2 * 3 + 1] * c[3];
-			tb66 = in[2 * 6 + 1] * c[6] + in[2 * 0 + 1];
-
-			{
-				real tmp1a, tmp2a, tmp1b, tmp2b;
-				tmp1a =	in[2 * 1 + 0] * c[1] + ta33 + in[2 * 5 + 0] * c[5] + in[2 * 7 + 0] * c[7];
-				tmp1b =	in[2 * 1 + 1] * c[1] + tb33 + in[2 * 5 + 1] * c[5] + in[2 * 7 + 1] * c[7];
-				tmp2a =	in[2 * 2 + 0] * c[2] + in[2 * 4 + 0] * c[4] + ta66 + in[2 * 8 + 0] * c[8];
-				tmp2b =	in[2 * 2 + 1] * c[2] + in[2 * 4 + 1] * c[4] + tb66 + in[2 * 8 + 1] * c[8];
-
-				MACRO1(0);
-				MACRO2(8);
-			}
-
-			{
-				real tmp1a, tmp2a, tmp1b, tmp2b;
-				tmp1a =	(in[2 * 1 + 0] - in[2 * 5 + 0] - in[2 * 7 + 0]) * c[3];
-				tmp1b =	(in[2 * 1 + 1] - in[2 * 5 + 1] - in[2 * 7 + 1]) * c[3];
-				tmp2a =	(in[2 * 2 + 0] - in[2 * 4 + 0] - in[2 * 8 + 0]) * c[6] - in[2 * 6 + 0] + in[2 * 0 + 0];
-				tmp2b =	(in[2 * 2 + 1] - in[2 * 4 + 1] - in[2 * 8 + 1]) * c[6] - in[2 * 6 + 1] + in[2 * 0 + 1];
-
-				MACRO1(1);
-				MACRO2(7);
-			}
-
-			{
-				real tmp1a, tmp2a, tmp1b, tmp2b;
-				tmp1a =	in[2 * 1 + 0] * c[5] - ta33 - in[2 * 5 + 0] * c[7] + in[2 * 7 + 0] * c[1];
-				tmp1b =	in[2 * 1 + 1] * c[5] - tb33 - in[2 * 5 + 1] * c[7] + in[2 * 7 + 1] * c[1];
-				tmp2a =	-in[2 * 2 + 0] * c[8] - in[2 * 4 + 0] * c[2] + ta66 + in[2 * 8 + 0] * c[4];
-				tmp2b =	-in[2 * 2 + 1] * c[8] - in[2 * 4 + 1] * c[2] + tb66 + in[2 * 8 + 1] * c[4];
-
-				MACRO1(2);
-				MACRO2(6);
-			}
-
-			{
-				real tmp1a, tmp2a, tmp1b, tmp2b;
-				tmp1a =	in[2 * 1 + 0] * c[7] - ta33 + in[2 * 5 + 0] * c[1] - in[2 * 7 + 0] * c[5];
-				tmp1b =	in[2 * 1 + 1] * c[7] - tb33 + in[2 * 5 + 1] * c[1] - in[2 * 7 + 1] * c[5];
-				tmp2a =	-in[2 * 2 + 0] * c[4] + in[2 * 4 + 0] * c[8] + ta66 - in[2 * 8 + 0] * c[2];
-				tmp2b =	-in[2 * 2 + 1] * c[4] + in[2 * 4 + 1] * c[8] + tb66 - in[2 * 8 + 1] * c[2];
-
-				MACRO1(3);
-				MACRO2(5);
-			}
-
-			{
-				real sum0, sum1;
-
-				sum0 = in[2 * 0 + 0] - in[2 * 2 + 0] + in[2 * 4 + 0] - in[2 * 6 + 0] + in[2 * 8 + 0];
-				sum1 = (in[2 * 0 + 1] - in[2 * 2 + 1] + in[2 * 4 + 1] - in[2 * 6 + 1] + in[2 * 8 + 1]) * tfcos36[4];
-				MACRO0(4);
-			}
-		}
+#ifndef USE_SIMD
+static
 #endif
+void mpg123_dct36(real * inbuf, real * o1, real * o2, real * wintab, real * tsbuf)
+{
+	real tmp[18];
+	register real *in = inbuf;
 
+	in[17] += in[16];
+	in[16] += in[15];
+	in[15] += in[14];
+	in[14] += in[13];
+	in[13] += in[12];
+	in[12] += in[11];
+	in[11] += in[10];
+	in[10] += in[9];
+	in[9] += in[8];
+	in[8] += in[7];
+	in[7] += in[6];
+	in[6] += in[5];
+	in[5] += in[4];
+	in[4] += in[3];
+	in[3] += in[2];
+	in[2] += in[1];
+	in[1] += in[0];
+
+	in[17] += in[15];
+	in[15] += in[13];
+	in[13] += in[11];
+	in[11] += in[9];
+	in[9] += in[7];
+	in[7] += in[5];
+	in[5] += in[3];
+	in[3] += in[1];
+
+
+	{
+		real t3;
+		{
+			real t0, t1, t2;
+
+			t0 = COS6_2 * (in[8] + in[16] - in[4]);
+			t1 = COS6_2 * in[12];
+
+			t3 = in[0];
+			t2 = t3 - t1 - t1;
+			tmp[1] = tmp[7] = t2 - t0;
+			tmp[4] = t2 + t0 + t0;
+			t3 += t1;
+
+			t2 = COS6_1 * (in[10] + in[14] - in[2]);
+			tmp[1] -= t2;
+			tmp[7] += t2;
+		}
+		{
+			real t0, t1, t2;
+
+			t0 = cos9[0] * (in[4] + in[8]);
+			t1 = cos9[1] * (in[8] - in[16]);
+			t2 = cos9[2] * (in[4] + in[16]);
+
+			tmp[2] = tmp[6] = t3 - t0 - t2;
+			tmp[0] = tmp[8] = t3 + t0 + t1;
+			tmp[3] = tmp[5] = t3 - t1 + t2;
+		}
+	}
+	{
+		real t1, t2, t3;
+
+		t1 = cos18[0] * (in[2] + in[10]);
+		t2 = cos18[1] * (in[10] - in[14]);
+		t3 = COS6_1 * in[6];
+
+		{
+			real t0 = t1 + t2 + t3;
+			tmp[0] += t0;
+			tmp[8] -= t0;
+		}
+
+		t2 -= t3;
+		t1 -= t3;
+
+		t3 = cos18[2] * (in[2] + in[14]);
+
+		t1 += t3;
+		tmp[3] += t1;
+		tmp[5] -= t1;
+
+		t2 -= t3;
+		tmp[2] += t2;
+		tmp[6] -= t2;
+	}
+
+	{
+		real t0, t1, t2, t3, t4, t5, t6, t7;
+
+		t1 = COS6_2 * in[13];
+		t2 = COS6_2 * (in[9] + in[17] - in[5]);
+
+		t3 = in[1] + t1;
+		t4 = in[1] - t1 - t1;
+		t5 = t4 - t2;
+
+		t0 = cos9[0] * (in[5] + in[9]);
+		t1 = cos9[1] * (in[9] - in[17]);
+
+		tmp[13] = (t4 + t2 + t2) * tfcos36[17 - 13];
+		t2 = cos9[2] * (in[5] + in[17]);
+
+		t6 = t3 - t0 - t2;
+		t0 += t3 + t1;
+		t3 += t2 - t1;
+
+		t2 = cos18[0] * (in[3] + in[11]);
+		t4 = cos18[1] * (in[11] - in[15]);
+		t7 = COS6_1 * in[7];
+
+		t1 = t2 + t4 + t7;
+		tmp[17] = (t0 + t1) * tfcos36[17 - 17];
+		tmp[9] = (t0 - t1) * tfcos36[17 - 9];
+		t1 = cos18[2] * (in[3] + in[15]);
+		t2 += t1 - t7;
+
+		tmp[14] = (t3 + t2) * tfcos36[17 - 14];
+		t0 = COS6_1 * (in[11] + in[15] - in[3]);
+		tmp[12] = (t3 - t2) * tfcos36[17 - 12];
+
+		t4 -= t1 + t7;
+
+		tmp[16] = (t5 - t0) * tfcos36[17 - 16];
+		tmp[10] = (t5 + t0) * tfcos36[17 - 10];
+		tmp[15] = (t6 + t4) * tfcos36[17 - 15];
+		tmp[11] = (t6 - t4) * tfcos36[17 - 11];
+	}
+
+	{
+		register real *out2 = o2;
+		register real *w = wintab;
+		register real *out1 = o1;
+		register real *ts = tsbuf;
+
+		MACRO(0);
+		MACRO(1);
+		MACRO(2);
+		MACRO(3);
+		MACRO(4);
+		MACRO(5);
+		MACRO(6);
+		MACRO(7);
+		MACRO(8);
 	}
 }
 
@@ -1863,13 +1718,8 @@ static void III_hybrid(real fsIn[SBLIMIT][SSLIMIT],
 	if (gr_info->mixed_block_flag)
 	{
 		sb = 2;
-#ifdef USE_3DNOW
-	     (fr->dct36)(fsIn[0],rawout1,rawout2,win[0],tspnt);
-	     (fr->dct36)(fsIn[1],rawout1+18,rawout2+18,win1[0],tspnt+1);
-#else
-		dct36(fsIn[0], rawout1, rawout2, win[0], tspnt);
-		dct36(fsIn[1], rawout1 + 18, rawout2 + 18, win1[0], tspnt + 1);
-#endif
+		DCT36(fsIn[0],rawout1,rawout2,win[0],tspnt);
+		DCT36(fsIn[1],rawout1+18,rawout2+18,win1[0],tspnt+1);
 		rawout1 += 36;
 		rawout2 += 36;
 		tspnt += 2;
@@ -1888,13 +1738,8 @@ static void III_hybrid(real fsIn[SBLIMIT][SSLIMIT],
 	{
 		for (; sb < gr_info->maxb; sb += 2, tspnt += 2, rawout1 += 36, rawout2 += 36)
 		{
-#ifdef USE_3DNOW
-			(fr->dct36)(fsIn[sb],rawout1,rawout2,win[bt],tspnt);
-			(fr->dct36)(fsIn[sb+1],rawout1+18,rawout2+18,win1[bt],tspnt+1);
-#else
-			dct36(fsIn[sb], rawout1, rawout2, win[bt], tspnt);
-			dct36(fsIn[sb + 1], rawout1 + 18, rawout2 + 18, win1[bt], tspnt + 1);
-#endif
+			DCT36(fsIn[sb], rawout1, rawout2, win[bt], tspnt);
+			DCT36(fsIn[sb+1], rawout1+18, rawout2+18, win1[bt], tspnt+1);
 		}
 	}
 
@@ -2052,6 +1897,21 @@ int mpg123_do_layer3(struct frame *fr)
 			}
 		}
 
+#ifdef USE_SIMD
+		if (fr->synth_type == SYNTH_MMX && single < 0)
+		{
+			int i, sb;
+
+			for (sb = 0, i = 0; sb < SBLIMIT; sb++)
+			{
+				for (ss = 0; ss < SSLIMIT; ss++)
+				{
+					hybridIn[0][sb][ss] *= 16384.0;
+					hybridIn[1][sb][ss] *= 16384.0;
+				}
+			}
+		}
+#endif
 		for (ch = 0; ch < stereo1; ch++)
 		{
 			struct gr_info_s *gr_info = &(sideinfo.ch[ch].gr[gr]);
