@@ -5,6 +5,9 @@
 #include <stdlib.h>
 #include <pthread.h>
 
+#define CPU_HAS_MMX() (cpu_fflags & 0x800000)
+#define CPU_HAS_3DNOW() (cpu_efflags & 0x80000000)
+
 static const long outscale = 32768;
 
 static struct frame fr, temp_fr;
@@ -15,11 +18,12 @@ static pthread_t decode_thread;
 static gboolean audio_error = FALSE, output_opened = FALSE, dopause = FALSE;
 gint mpg123_bitrate, mpg123_frequency, mpg123_length, mpg123_layer, mpg123_lsf;
 gchar *mpg123_title = NULL, *mpg123_filename = NULL;
-static gint disp_bitrate, skip_frames = 0;
+static int disp_bitrate, skip_frames = 0;
+static int cpu_fflags, cpu_efflags;
 gboolean mpg123_stereo, mpg123_mpeg25;
-gint mpg123_mode;
+int mpg123_mode;
 
-const gchar *mpg123_id3_genres[GENRE_MAX] =
+const char *mpg123_id3_genres[GENRE_MAX] =
 {
 	N_("Blues"), N_("Classic Rock"), N_("Country"), N_("Dance"),
 	N_("Disco"), N_("Funk"), N_("Grunge"), N_("Hip-Hop"),
@@ -30,7 +34,7 @@ const gchar *mpg123_id3_genres[GENRE_MAX] =
 	N_("Euro-Techno"), N_("Ambient"), N_("Trip-Hop"), N_("Vocal"),
 	N_("Jazz+Funk"), N_("Fusion"), N_("Trance"), N_("Classical"),
 	N_("Instrumental"), N_("Acid"), N_("House"), N_("Game"),
-	N_("Sound Clip"), N_("Gospel"), N_("Noise"), N_("Alt"),
+	N_("Sound Clip"), N_("Gospel"), N_("Noise"), N_("AlternRock"),
 	N_("Bass"), N_("Soul"), N_("Punk"), N_("Space"),
 	N_("Meditative"), N_("Instrumental Pop"),
 	N_("Instrumental Rock"), N_("Ethnic"), N_("Gothic"),
@@ -76,83 +80,70 @@ double mpg123_compute_tpf(struct frame *fr)
 	return tpf;
 }
 
-void set_mpg123_synth_functions(struct frame *fr)
+static void set_synth_functions(struct frame *fr)
 {
 	typedef int (*func) (real *, int, unsigned char *, int *);
 	typedef int (*func_mono) (real *, unsigned char *, int *);
-#ifdef USE_3DNOW
-	typedef void (*func_dct36)(real *,real *,real *,real *,real *);
-#endif
+	typedef void (*func_dct36)(real *, real *, real *, real *, real *);
+
 	int ds = fr->down_sample;
 	int p8 = 0;
 
-#ifdef USE_3DNOW
-	static func funcs[3][4] =
+	static func funcs[][3] =
 	{
-#else
-	static func funcs[2][4] =
-	{
-#endif
 		{mpg123_synth_1to1,
 		 mpg123_synth_2to1,
-		 mpg123_synth_4to1,
-		 mpg123_synth_ntom},
+		 mpg123_synth_4to1},
 		{mpg123_synth_1to1_8bit,
 		 mpg123_synth_2to1_8bit,
-		 mpg123_synth_4to1_8bit,
-		 mpg123_synth_ntom_8bit}
-#ifdef USE_3DNOW
-  	       ,{ mpg123_synth_1to1_3dnow,
-  		  mpg123_synth_2to1,
- 		  mpg123_synth_4to1,
-  		  mpg123_synth_ntom }
+		 mpg123_synth_4to1_8bit},
+#ifdef USE_SIMD
+		{mpg123_synth_1to1_mmx,
+		 mpg123_synth_2to1,
+		 mpg123_synth_4to1},
+		{mpg123_synth_1to1_3dnow,
+		 mpg123_synth_2to1,
+		 mpg123_synth_4to1}
 #endif
 	};
 
-	static func_mono funcs_mono[2][2][4] =
+	static func_mono funcs_mono[2][4] =
 	{
-		{
-			{mpg123_synth_1to1_mono2stereo,
-			 mpg123_synth_2to1_mono2stereo,
-			 mpg123_synth_4to1_mono2stereo,
-			 mpg123_synth_ntom_mono2stereo},
-			{mpg123_synth_1to1_8bit_mono2stereo,
-			 mpg123_synth_2to1_8bit_mono2stereo,
-			 mpg123_synth_4to1_8bit_mono2stereo,
-			 mpg123_synth_ntom_8bit_mono2stereo}},
-		{
-			{mpg123_synth_1to1_mono,
-			 mpg123_synth_2to1_mono,
-			 mpg123_synth_4to1_mono,
-			 mpg123_synth_ntom_mono},
-			{mpg123_synth_1to1_8bit_mono,
-			 mpg123_synth_2to1_8bit_mono,
-			 mpg123_synth_4to1_8bit_mono,
-			 mpg123_synth_ntom_8bit_mono}}
+		{mpg123_synth_1to1_mono,
+		 mpg123_synth_2to1_mono,
+		 mpg123_synth_4to1_mono},
+		{mpg123_synth_1to1_8bit_mono,
+		 mpg123_synth_2to1_8bit_mono,
+		 mpg123_synth_4to1_8bit_mono}
 	};
 
-#ifdef USE_3DNOW	
-	static func_dct36 funcs_dct36[2] = {dct36 , dct36_3dnow};
+#ifdef USE_SIMD
+	static func_dct36 funcs_dct36[2] = {mpg123_dct36, dct36_3dnow};
 #endif
 
 	if (mpg123_cfg.resolution == 8)
 		p8 = 1;
 	fr->synth = funcs[p8][ds];
-	fr->synth_mono = funcs_mono[1][p8][ds];
+	fr->synth_mono = funcs_mono[p8][ds];
+	fr->synth_type = SYNTH_FPU;
 
-#ifdef USE_3DNOW
-       /* check cpuflags bit 31 (3DNow!) */
-	if( (mpg123_cfg.use_3dnow < 2 ) && 
-	    ((mpg123_cfg.use_3dnow == 1 ) ||
-	     (support_3dnow() == TRUE)) &&
-	    (mpg123_cfg.resolution != 8) )
+#ifdef USE_SIMD
+	fr->dct36 = funcs_dct36[0];
+   
+	if (CPU_HAS_3DNOW() && !p8 &&
+	    (mpg123_cfg.default_synth == SYNTH_3DNOW || 
+	     mpg123_cfg.default_synth == SYNTH_AUTO))
 	{
-		fr->synth = funcs[2][ds]; /* 3DNow! optimized synth_1to1() */
+		fr->synth = funcs[3][ds]; /* 3DNow! optimized synth_1to1() */
 		fr->dct36 = funcs_dct36[1]; /* 3DNow! optimized dct36() */
+		fr->synth_type = SYNTH_3DNOW;
 	}
-	else
+	else if (CPU_HAS_MMX() && !p8 &&
+		 (mpg123_cfg.default_synth == SYNTH_MMX ||
+		  mpg123_cfg.default_synth == SYNTH_AUTO))
 	{
-		fr->dct36 = funcs_dct36[0];
+		fr->synth = funcs[2][ds]; /* MMX optimized synth_1to1() */
+		fr->synth_type = SYNTH_MMX;
 	}
 #endif
 	if (p8)
@@ -170,7 +161,6 @@ static void init(void)
 	mpg123_cfg.resolution = 16;
 	mpg123_cfg.channels = 2;
 	mpg123_cfg.downsample = 0;
-	mpg123_cfg.downsample_custom = 44100;
 	mpg123_cfg.http_buffer_size = 128;
 	mpg123_cfg.http_prebuffer = 25;
 	mpg123_cfg.proxy_port = 8080;
@@ -181,15 +171,14 @@ static void init(void)
 	mpg123_cfg.use_udp_channel = TRUE;
 	mpg123_cfg.title_override = FALSE;
 	mpg123_cfg.disable_id3v2 = FALSE;
-	mpg123_cfg.detect_by_content = FALSE;
-	mpg123_cfg.use_3dnow = 0;
+	mpg123_cfg.detect_by = DETECT_EXTENSION;
+	mpg123_cfg.default_synth = SYNTH_AUTO;
 
 	cfg = xmms_cfg_open_default_file();
 
 	xmms_cfg_read_int(cfg, "MPG123", "resolution", &mpg123_cfg.resolution);
 	xmms_cfg_read_int(cfg, "MPG123", "channels", &mpg123_cfg.channels);
 	xmms_cfg_read_int(cfg, "MPG123", "downsample", &mpg123_cfg.downsample);
-	/*xmms_cfg_read_int(cfg,"MPG123","downsample_custom",&mpg123_cfg.downsample_custom); */
 	xmms_cfg_read_int(cfg, "MPG123", "http_buffer_size", &mpg123_cfg.http_buffer_size);
 	xmms_cfg_read_int(cfg, "MPG123", "http_prebuffer", &mpg123_cfg.http_prebuffer);
 	xmms_cfg_read_boolean(cfg, "MPG123", "save_http_stream", &mpg123_cfg.save_http_stream);
@@ -210,10 +199,16 @@ static void init(void)
 	xmms_cfg_read_boolean(cfg, "MPG123", "disable_id3v2", &mpg123_cfg.disable_id3v2);
 	if (!xmms_cfg_read_string(cfg, "MPG123", "id3_format", &mpg123_cfg.id3_format))
 		mpg123_cfg.id3_format = g_strdup("%p - %t");
-	xmms_cfg_read_boolean(cfg, "MPG123", "detect_by_content", &mpg123_cfg.detect_by_content);
-	xmms_cfg_read_int(cfg, "MPG123", "use_3dnow", &mpg123_cfg.use_3dnow);
+	xmms_cfg_read_int(cfg, "MPG123", "detect_by", &mpg123_cfg.detect_by);
+	xmms_cfg_read_int(cfg, "MPG123", "default_synth", &mpg123_cfg.default_synth);
 
 	xmms_cfg_free(cfg);
+
+	if (mpg123_cfg.resolution != 16 && mpg123_cfg.resolution != 8)
+		mpg123_cfg.resolution = 16;
+	mpg123_cfg.channels = CLAMP(mpg123_cfg.channels, 0, 2);
+	mpg123_cfg.downsample = CLAMP(mpg123_cfg.downsample, 0, 2);
+	mpg123_getcpuflags(&cpu_fflags, &cpu_efflags);
 }
 
 /* needed for is_our_file() */
@@ -329,21 +324,22 @@ static guint16 read_wav_id(char *filename)
 
 #define DET_BUF_SIZE 1024
 
-static gboolean mpg123_detect_by_content(gchar *filename)
+static gboolean mpg123_detect_by_content(char *filename)
 {
 	FILE *file;
 	guchar tmp[4];
 	guint32 head;
 	struct frame fr;
 	guchar buf[DET_BUF_SIZE];
-	gint in_buf, i;
+	int in_buf, i;
+	gboolean ret = FALSE;
 
-	if((file = fopen(filename, "rb")) == NULL)
+	if ((file = fopen(filename, "rb")) == NULL)
 		return FALSE;
 	if (fread(tmp, 1, 4, file) != 4)
 		goto done;
 	head = convert_to_header(tmp);
-	while(!mpg123_head_check(head))
+	while (!mpg123_head_check(head))
 	{
 		/*
 		 * The mpeg-stream can start anywhere in the file,
@@ -351,16 +347,16 @@ static gboolean mpg123_detect_by_content(gchar *filename)
 		 */
 		/* Optimize this */
 		in_buf = fread(buf, 1, DET_BUF_SIZE, file);
-		if(in_buf == 0)
+		if (in_buf == 0)
 			goto done;
 
 		for (i = 0; i < in_buf; i++)
 		{
 			head <<= 8;
 			head |= buf[i];
-			if(mpg123_head_check(head))
+			if (mpg123_head_check(head))
 			{
-				fseek(file, i+1-in_buf, SEEK_CUR);
+				fseek(file, i + 1 - in_buf, SEEK_CUR);
 				break;
 			}
 		}
@@ -378,15 +374,12 @@ static gboolean mpg123_detect_by_content(gchar *filename)
 			goto done;
 		head = convert_to_header(tmp);
 		if (mpg123_head_check(head) && mpg123_decode_header(&fr, head))
-		{
-			fclose(file);
-			return TRUE;
-		}
+			ret = TRUE;
 	}
 
  done:
 	fclose(file);
-	return FALSE;
+	return ret;
 }
 
 static int is_our_file(char *filename)
@@ -409,8 +402,9 @@ static int is_our_file(char *filename)
 		}
 		return TRUE;
 	}
-	if(mpg123_cfg.detect_by_content)
+	if(mpg123_cfg.detect_by == DETECT_CONTENT)
 		return (mpg123_detect_by_content(filename));
+
 	ext = strrchr(filename, '.');
 	if (ext)
 	{
@@ -427,6 +421,9 @@ static int is_our_file(char *filename)
 			}
 		}
 	}
+
+	if(mpg123_cfg.detect_by == DETECT_BOTH)
+		return (mpg123_detect_by_content(filename));
 	return FALSE;
 }
 
@@ -552,7 +549,7 @@ static char* mpg123_getstr(char* str)
  */
 gchar *mpg123_format_song_title(struct id3tag_t *tag, gchar * filename)
 {
-	gchar *ret = NULL;
+	gchar *ret = NULL, *path, *temp;
 	TitleInput *input;
 
 	XMMS_NEW_TITLEINPUT(input);
@@ -567,13 +564,18 @@ gchar *mpg123_format_song_title(struct id3tag_t *tag, gchar * filename)
 		input->genre = mpg123_getstr(tag->genre);
 		input->comment = mpg123_getstr(tag->comment);
 	}
+	path = g_strdup(filename);
+	temp = strrchr(path, '/');
+	if (temp)
+		*temp = '\0';
 	input->file_name = g_basename(filename);
-	input->file_path = filename;
+	input->file_path = g_strdup_printf("%s/", path);
 	input->file_ext = extname(filename);
 	ret = xmms_get_titlestring(mpg123_cfg.title_override ?
 				   mpg123_cfg.id3_format :
 				   xmms_get_gentitle_format(), input);
 	g_free(input);
+	g_free(path);
 
 	if (!ret)
 	{
@@ -595,9 +597,9 @@ gchar *mpg123_format_song_title(struct id3tag_t *tag, gchar * filename)
  *    `tag'. 
  *
  */
-void mpg123_get_id3v2(id3_t * id3d, struct id3tag_t *tag)
+void mpg123_get_id3v2(struct id3_tag *id3d, struct id3tag_t *tag)
 {
-	id3_frame_t *id3frm;
+	struct id3_frame *id3frm;
 	gchar *txt;
 	gint tlen, num;
 
@@ -640,7 +642,7 @@ void mpg123_get_id3v2(id3_t * id3d, struct id3tag_t *tag)
 	ID3_SET		(ID3_TALB, album);
 	ID3_SET_NUM	(ID3_TYER, year);
 	ID3_SET_NUM	(ID3_TRCK, track_number);
-	ID3_SET		(ID3_TXXX, comment);
+	ID3_SET		(ID3_COMM, comment);
 	ID3_SET		(ID3_TCON, genre);
 }
 
@@ -662,7 +664,7 @@ static gchar *get_song_title(FILE * fd, char *filename)
 
 	if (file || (file = fopen(filename, "rb")) != 0)
 	{
-		id3_t *id3 = NULL;
+		struct id3_tag *id3 = NULL;
 
 		/*
 		 * Try reading ID3v2 tag.
@@ -706,6 +708,21 @@ static gchar *get_song_title(FILE * fd, char *filename)
 	return ret;
 }
 
+static long get_song_length(FILE *file)
+{
+	int len;
+	char tmp[4];
+
+	fseek(file, 0, SEEK_END);
+	len = ftell(file);
+	fseek(file, -128, SEEK_END);
+	fread(tmp, 1, 3, file);
+	if (!strncmp(tmp, "TAG", 3))
+		len -= 128;
+	return len;
+}
+
+
 static guint get_song_time(FILE * file)
 {
 	guint32 head;
@@ -738,17 +755,14 @@ static guint get_song_time(FILE * file)
 		if (mpg123_get_xing_header(&xing_header, buf))
 		{
 			g_free(buf);
-			return ((guint) (tpf * xing_header.frames * 1000));
+			if (xing_header.bytes == 0)
+				xing_header.bytes = get_song_length(file);
+			return (tpf * xing_header.frames * 1000);
 		}
 		g_free(buf);
 		bpf = mpg123_compute_bpf(&frm);
-		fseek(file, 0, SEEK_END);
-		len = ftell(file);
-		fseek(file, -128, SEEK_END);
-		fread(tmp, 1, 3, file);
-		if (!strncmp(tmp, "TAG", 3))
-			len -= 128;
-		return ((guint) ((guint)(len / bpf) * tpf * 1000));
+		len = get_song_length(file);
+		return ((guint)(len / bpf) * tpf * 1000);
 	}
 	return 0;
 }
@@ -791,11 +805,39 @@ static int open_output(void)
 	return r;
 }
 
+
+static int mpg123_seek(struct frame *fr, xing_header_t *xh, gboolean vbr, int time)
+{
+	int jumped = -1;
+	
+	if (xh)
+	{
+		int percent = ((double) time * 100.0) /
+			(mpg123_info->num_frames * mpg123_info->tpf);
+		int byte = mpg123_seek_point(xh, percent);
+		jumped = mpg123_stream_jump_to_byte(fr, byte);
+	}
+	else if (vbr && mpg123_length > 0)
+	{
+		int byte = ((guint64)time * 1000 * mpg123_info->filesize) /
+			mpg123_length;
+		jumped = mpg123_stream_jump_to_byte(fr, byte);
+	}
+	else
+	{
+		int frame = time / mpg123_info->tpf;
+		jumped = mpg123_stream_jump_to_frame(fr, frame);
+	}
+
+	return jumped;
+}
+
+
 static void *decode_loop(void *arg)
 {
 	gboolean have_xing_header = FALSE, vbr = FALSE;
-	gint disp_count = 0, temp_time;
-	gchar *filename = arg;
+	int disp_count = 0, temp_time;
+	char *filename = arg;
 	xing_header_t xing_header;
 
 	/* This is used by fileinfo on http streams */
@@ -819,7 +861,7 @@ static void *decode_loop(void *arg)
 
 		fr.down_sample = mpg123_cfg.downsample;
 		fr.down_sample_sblimit = SBLIMIT >> mpg123_cfg.downsample;
-		set_mpg123_synth_functions(&fr);
+		set_synth_functions(&fr);
 		mpg123_init_layer3(fr.down_sample_sblimit);
 
 		mpg123_info->tpf = mpg123_compute_tpf(&fr);
@@ -833,9 +875,9 @@ static void *decode_loop(void *arg)
 			}
 		}
 
-		for(;;)
+		for (;;)
 		{
-			memcpy(&temp_fr,&fr,sizeof(struct frame));
+			memcpy(&temp_fr, &fr, sizeof(struct frame));
 			if (!mpg123_read_frame(&temp_fr))
 			{
 				mpg123_info->eof = TRUE;
@@ -848,13 +890,13 @@ static void *decode_loop(void *arg)
 			else
 				break;
 		}
-		if(!have_xing_header && strncasecmp(filename, "http://", 7))
-		{
-			mpg123_info->num_frames = mpg123_calc_numframes(&fr);
-		}
 
-		memcpy(&fr,&temp_fr,sizeof(struct frame));
-		mpg123_bitrate = disp_bitrate = tabsel_123[fr.lsf][fr.lay - 1][fr.bitrate_index];
+		if (!have_xing_header && strncasecmp(filename, "http://", 7))
+			mpg123_info->num_frames = mpg123_calc_numframes(&fr);
+
+		memcpy(&fr, &temp_fr, sizeof(struct frame));
+		mpg123_bitrate = tabsel_123[fr.lsf][fr.lay - 1][fr.bitrate_index];
+		disp_bitrate = mpg123_bitrate;
 		mpg123_frequency = mpg123_freqs[fr.sampling_frequency];
 		mpg123_stereo = fr.stereo;
 		mpg123_layer = fr.lay;
@@ -864,7 +906,8 @@ static void *decode_loop(void *arg)
 
 		if (strncasecmp(filename, "http://", 7))
 		{
-			mpg123_length = mpg123_info->num_frames * mpg123_info->tpf * 1000;
+			mpg123_length =
+				mpg123_info->num_frames * mpg123_info->tpf * 1000;
 			if (!mpg123_title)
 				mpg123_title = get_song_title(NULL,filename);
 		}
@@ -874,7 +917,10 @@ static void *decode_loop(void *arg)
 				mpg123_title = mpg123_http_get_title(filename);
 			mpg123_length = -1;
 		}
-		mpg123_ip.set_info(mpg123_title, mpg123_length, mpg123_bitrate * 1000, mpg123_freqs[fr.sampling_frequency], fr.stereo);
+		mpg123_ip.set_info(mpg123_title, mpg123_length,
+				   mpg123_bitrate * 1000,
+				   mpg123_freqs[fr.sampling_frequency],
+				   fr.stereo);
 		output_opened = TRUE;
 		if (!open_output())
 		{
@@ -890,22 +936,16 @@ static void *decode_loop(void *arg)
 	{
 		if (mpg123_info->jump_to_time != -1)
 		{
-			int jumped = -1;
-
+			void *xp = NULL;
 			if (have_xing_header)
-				jumped = mpg123_stream_jump_to_byte(&fr, mpg123_seek_point(&xing_header, ((double) mpg123_info->jump_to_time * 100.0) / (mpg123_info->num_frames * mpg123_info->tpf)));
-			else if (vbr && mpg123_length > 0)
-				jumped = mpg123_stream_jump_to_byte(&fr, ((guint64)mpg123_info->jump_to_time * 1000 * mpg123_info->filesize) / mpg123_length);
-			else
-				jumped = mpg123_stream_jump_to_frame(&fr, mpg123_info->jump_to_time / mpg123_info->tpf);
-
-			if (jumped > -1)
+				xp = &xing_header;
+			if (mpg123_seek(&fr, xp, vbr,
+					mpg123_info->jump_to_time) > -1)
 			{
 				mpg123_ip.output->flush(mpg123_info->jump_to_time * 1000);
 				mpg123_info->eof = FALSE;
 			}
 			mpg123_info->jump_to_time = -1;
-
 		}
 		if (!mpg123_info->eof)
 		{
@@ -913,7 +953,7 @@ static void *decode_loop(void *arg)
 			{
 				if(fr.lay != mpg123_layer || fr.lsf != mpg123_lsf)
 				{
-					memcpy(&temp_fr,&fr,sizeof(struct frame));
+					memcpy(&temp_fr, &fr, sizeof(struct frame));
 					if(mpg123_read_frame(&temp_fr) != 0)
 					{
 						if(fr.lay == temp_fr.lay && fr.lsf == temp_fr.lsf)
@@ -921,7 +961,6 @@ static void *decode_loop(void *arg)
 							mpg123_layer = fr.lay;
 							mpg123_lsf = fr.lsf;
 							memcpy(&fr,&temp_fr,sizeof(struct frame));
-							set_mpg123_synth_functions(&fr);
 						}
 						else
 						{
@@ -960,7 +999,6 @@ static void *decode_loop(void *arg)
 							mpg123_ip.output->flush(temp_time);
 							mpg123_ip.set_info(mpg123_title, mpg123_length, mpg123_bitrate * 1000, mpg123_frequency, mpg123_stereo);
 							memcpy(&fr,&temp_fr,sizeof(struct frame));
-							set_mpg123_synth_functions(&fr);
 						}
 						else
 						{
@@ -980,6 +1018,7 @@ static void *decode_loop(void *arg)
 					disp_count = 20;
 					if (mpg123_bitrate != disp_bitrate)
 					{
+						/* FIXME networks streams */
 						disp_bitrate = mpg123_bitrate;
 						if(!have_xing_header && strncasecmp(filename,"http://",7))
 						{
