@@ -1,6 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-2001  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999-2001  Hvard Kvlen
+ *  Copyright (C) 1998-2002  Peter Alm, Mikael Alm, Olle Hallnas,
+ *                           Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1999-2002  Haavard Kvaalen
  *  Copyright (C) 2001, Jorn Baayen <jorn@nl.linux.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -60,6 +61,9 @@ static GtkWidget *genre_combo, *user_comment_entry;
 static GtkWidget *description_entry, *version_entry, *isrc_entry;
 static GtkWidget *copyright_entry, *organization_entry, *location_entry;
 #endif
+static GtkWidget *rg_track_entry, *rg_album_entry, *rg_track_peak_entry, *rg_album_peak_entry;
+static GtkWidget *rg_track_label, *rg_album_label, *rg_track_peak_label, *rg_album_peak_label;
+static GtkWidget *rg_show_button;
 
 /* From mpg123.c, as no standardized Ogg Vorbis genres exists. */
 static const gchar *vorbis_genres[] =
@@ -73,7 +77,7 @@ static const gchar *vorbis_genres[] =
 	N_("Euro-Techno"), N_("Ambient"), N_("Trip-Hop"), N_("Vocal"),
 	N_("Jazz+Funk"), N_("Fusion"), N_("Trance"), N_("Classical"),
 	N_("Instrumental"), N_("Acid"), N_("House"), N_("Game"),
-	N_("Sound Clip"), N_("Gospel"), N_("Noise"), N_("Alt"),
+	N_("Sound Clip"), N_("Gospel"), N_("Noise"), N_("AlternRock"),
 	N_("Bass"), N_("Soul"), N_("Punk"), N_("Space"),
 	N_("Meditative"), N_("Instrumental Pop"),
 	N_("Instrumental Rock"), N_("Ethnic"), N_("Gothic"),
@@ -214,6 +218,7 @@ static void save_cb(GtkWidget * w, gpointer data)
 	gchar *description, *version, *isrc, *copyright, *organization;
 	gchar *location;
 #endif
+	gchar *rg_track_gain, *rg_album_gain, *rg_track_peak, *rg_album_peak;
 	char **comment_list;
 	vcedit_state *state;
 	vorbis_comment *comment;
@@ -251,6 +256,10 @@ static void save_cb(GtkWidget * w, gpointer data)
 	organization = gtk_entry_get_text(GTK_ENTRY(organization_entry));
 	copyright = gtk_entry_get_text(GTK_ENTRY(copyright_entry));
 #endif
+	rg_track_gain = gtk_entry_get_text(GTK_ENTRY(rg_track_entry));
+	rg_album_gain = gtk_entry_get_text(GTK_ENTRY(rg_album_entry));
+	rg_track_peak = gtk_entry_get_text(GTK_ENTRY(rg_track_peak_entry));
+	rg_album_peak = gtk_entry_get_text(GTK_ENTRY(rg_album_peak_entry));
 
 	comment_list = add_tag(comment_list, "title", track_name);
 	comment_list = add_tag(comment_list, "artist", performer);
@@ -258,7 +267,7 @@ static void save_cb(GtkWidget * w, gpointer data)
 	comment_list = add_tag(comment_list, "tracknumber", track_number);
 	comment_list = add_tag(comment_list, "genre", genre);
 	comment_list = add_tag(comment_list, "date", date);
-	comment_list = add_tag(comment_list, "", user_comment); /* "" = user comment */
+	comment_list = add_tag(comment_list, "comment", user_comment);
 
 #ifdef ALL_VORBIS_TAGS
 	comment_list = add_tag(comment_list, "location", location);
@@ -268,6 +277,10 @@ static void save_cb(GtkWidget * w, gpointer data)
 	comment_list = add_tag(comment_list, "organization", organization);
 	comment_list = add_tag(comment_list, "copyright", copyright);
 #endif
+	comment_list = add_tag(comment_list, "replaygain_track_gain", rg_track_gain);
+	comment_list = add_tag(comment_list, "replaygain_album_gain", rg_album_gain);
+	comment_list = add_tag(comment_list, "replaygain_track_peak", rg_track_peak);
+	comment_list = add_tag(comment_list, "replaygain_album_peak", rg_album_peak);
 
 	add_list(comment, comment_list);
 	g_strfreev(comment_list);
@@ -311,6 +324,32 @@ close:
 	gtk_widget_destroy(window);
 }
 
+static void rg_show_cb(GtkWidget * w, gpointer data)
+{
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rg_show_button)))
+	{
+		gtk_widget_show(rg_track_label);
+		gtk_widget_show(rg_track_entry);
+		gtk_widget_show(rg_album_label);
+		gtk_widget_show(rg_album_entry);
+		gtk_widget_show(rg_track_peak_label);
+		gtk_widget_show(rg_track_peak_entry);
+		gtk_widget_show(rg_album_peak_label);
+		gtk_widget_show(rg_album_peak_entry);
+	}
+	else
+	{
+		gtk_widget_hide(rg_track_label);
+		gtk_widget_hide(rg_track_entry);
+		gtk_widget_hide(rg_album_label);
+		gtk_widget_hide(rg_album_entry);
+		gtk_widget_hide(rg_track_peak_label);
+		gtk_widget_hide(rg_track_peak_entry);
+		gtk_widget_hide(rg_album_peak_label);
+		gtk_widget_hide(rg_album_peak_entry);
+	}
+}
+
 static gint init_files(vcedit_state *state)
 {
 	if ((vte.in = fopen(vte.filename, "rb")) == NULL)
@@ -391,6 +430,7 @@ void vorbis_file_info_box(char *fn)
 	gchar *genre, *user_comment, *tmp;
 	gchar *description, *version, *isrc, *copyright, *organization;
 	gchar *location;
+	gchar *rg_track_gain, *rg_album_gain, *rg_track_peak, *rg_album_peak;
 
 	gint time, minutes, seconds, bitrate, rate, channels, filesize, i;
 
@@ -402,6 +442,7 @@ void vorbis_file_info_box(char *fn)
 
 	static GtkWidget *info_frame, *info_box, *bitrate_label, *rate_label;
 	static GtkWidget *channel_label, *length_label, *filesize_label;
+	static GtkWidget *replaygain_label, *audiophilegain_label, *peak_label;
 	static GtkWidget *filename_entry, *tag_frame;
 
 	g_free(vte.filename);
@@ -592,6 +633,55 @@ void vorbis_file_info_box(char *fn)
 				 GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);
 #endif
 
+		rg_show_button = gtk_check_button_new_with_label(_("ReplayGain Settings:"));
+		gtk_signal_connect(GTK_OBJECT(rg_show_button), "toggled",
+				   GTK_SIGNAL_FUNC(rg_show_cb), NULL);
+		gtk_table_attach(GTK_TABLE(table), rg_show_button, 0, 2, 11, 12,
+				 GTK_FILL, GTK_FILL, 5, 5);
+
+
+		rg_track_label = gtk_label_new(_("Track gain:"));
+		gtk_misc_set_alignment(GTK_MISC(rg_track_label), 1, 0.5);
+		gtk_table_attach(GTK_TABLE(table), rg_track_label, 2, 3, 11, 12,
+				 GTK_FILL, GTK_FILL, 5, 5);
+
+		rg_track_entry = gtk_entry_new();
+		gtk_table_attach(GTK_TABLE(table), rg_track_entry, 3, 4, 11,
+				 12, GTK_FILL |	GTK_EXPAND | GTK_SHRINK,
+				 GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);
+
+		rg_track_peak_label = gtk_label_new(_("Track peak:"));
+		gtk_misc_set_alignment(GTK_MISC(rg_track_peak_label), 1, 0.5);
+		gtk_table_attach(GTK_TABLE(table), rg_track_peak_label, 2, 3, 12, 13,
+				 GTK_FILL, GTK_FILL, 5, 5);
+
+		rg_track_peak_entry = gtk_entry_new();
+		gtk_table_attach(GTK_TABLE(table), rg_track_peak_entry, 3, 4, 12,
+				 13, GTK_FILL |	GTK_EXPAND | GTK_SHRINK,
+				 GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);
+
+
+		rg_album_label = gtk_label_new(_("Album gain:"));
+		gtk_misc_set_alignment(GTK_MISC(rg_album_label), 1, 0.5);
+		gtk_table_attach(GTK_TABLE(table), rg_album_label, 2, 3, 13, 14,
+				 GTK_FILL, GTK_FILL, 5, 5);
+
+		rg_album_entry = gtk_entry_new();
+		gtk_table_attach(GTK_TABLE(table), rg_album_entry, 3, 4, 13,
+				 14, GTK_FILL |	GTK_EXPAND | GTK_SHRINK,
+				 GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);
+
+		rg_album_peak_label = gtk_label_new(_("Album peak:"));
+		gtk_misc_set_alignment(GTK_MISC(rg_album_peak_label), 1, 0.5);
+		gtk_table_attach(GTK_TABLE(table), rg_album_peak_label, 2, 3, 14, 15,
+				 GTK_FILL, GTK_FILL, 5, 5);
+
+		rg_album_peak_entry = gtk_entry_new();
+		gtk_table_attach(GTK_TABLE(table), rg_album_peak_entry, 3, 4, 14,
+				 15, GTK_FILL |	GTK_EXPAND | GTK_SHRINK,
+				 GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 5);
+
+
 		bbox = gtk_hbutton_box_new(); 
 		gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox),
 					  GTK_BUTTONBOX_END);
@@ -661,6 +751,27 @@ void vorbis_file_info_box(char *fn)
 				      GTK_JUSTIFY_LEFT);
 		gtk_box_pack_start(GTK_BOX(info_box), filesize_label, FALSE,
 				   FALSE, 0);
+
+		replaygain_label = gtk_label_new("");
+		gtk_misc_set_alignment(GTK_MISC(replaygain_label), 0, 0);
+		gtk_label_set_justify(GTK_LABEL(replaygain_label),
+				      GTK_JUSTIFY_LEFT);
+		gtk_box_pack_start(GTK_BOX(info_box), replaygain_label, FALSE,
+				   FALSE, 0);
+
+		audiophilegain_label = gtk_label_new("");
+		gtk_misc_set_alignment(GTK_MISC(audiophilegain_label), 0, 0);
+		gtk_label_set_justify(GTK_LABEL(audiophilegain_label),
+				      GTK_JUSTIFY_LEFT);
+		gtk_box_pack_start(GTK_BOX(info_box), audiophilegain_label, FALSE,
+				   FALSE, 0);
+
+		peak_label = gtk_label_new("");
+		gtk_misc_set_alignment(GTK_MISC(peak_label), 0, 0);
+		gtk_label_set_justify(GTK_LABEL(peak_label),
+				      GTK_JUSTIFY_LEFT);
+		gtk_box_pack_start(GTK_BOX(info_box), peak_label, FALSE,
+				   FALSE, 0);
 		
 		gtk_widget_show_all(window);
 	} else
@@ -723,7 +834,7 @@ void vorbis_file_info_box(char *fn)
 	track_number = get_comment(comment, "tracknumber");
 	genre = get_comment(comment, "genre");
 	date = get_comment(comment, "date");
-	user_comment = get_comment(comment, ""); /* "" = user comment */
+	user_comment = get_comment(comment, "comment");
 	location = get_comment(comment, "location");
 	description = get_comment(comment, "description");
 	version = get_comment(comment, "version");
@@ -731,6 +842,25 @@ void vorbis_file_info_box(char *fn)
 	organization = get_comment(comment, "organization");
 	copyright = get_comment(comment, "copyright");
 
+	rg_track_gain = get_comment(comment, "replaygain_track_gain");
+	if (*rg_track_gain == '\0')
+	{
+		g_free(rg_track_gain);
+		rg_track_gain = get_comment(comment, "rg_radio"); /* Old */
+	}
+	rg_album_gain = get_comment(comment, "replaygain_album_gain");
+	if (*rg_album_gain == '\0')
+	{
+		g_free(rg_album_gain);
+		rg_album_gain = get_comment(comment, "rg_audiophile"); /* Old */
+	}
+	rg_track_peak = get_comment(comment, "replaygain_track_peak");
+	if (*rg_track_peak == '\0')
+	{
+		g_free(rg_track_peak);
+		rg_track_peak = get_comment(comment, "rg_peak"); /* Old */
+	}
+	rg_album_peak = get_comment(comment, "replaygain_album_peak"); /* Old had no album peak */
 
 	/* ov_clear closes the file */
 	if (clear_vf)
@@ -756,6 +886,21 @@ void vorbis_file_info_box(char *fn)
 	gtk_entry_set_text(GTK_ENTRY(filename_entry), vte.filename);
 	gtk_editable_set_position(GTK_EDITABLE(filename_entry), -1);
 
+	gtk_entry_set_text(GTK_ENTRY(rg_track_entry), rg_track_gain);
+	gtk_entry_set_text(GTK_ENTRY(rg_album_entry), rg_album_gain);
+	gtk_entry_set_text(GTK_ENTRY(rg_track_peak_entry), rg_track_peak);
+	gtk_editable_set_position(GTK_EDITABLE(rg_track_peak_entry), -1);
+	gtk_entry_set_text(GTK_ENTRY(rg_album_peak_entry), rg_album_peak);
+	gtk_editable_set_position(GTK_EDITABLE(rg_album_peak_entry), -1);
+
+	if (*rg_track_gain == '\0' && *rg_album_gain == '\0' && *rg_track_peak == '\0' && *rg_album_peak == '\0')
+	{
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rg_show_button), FALSE);
+		rg_show_cb(rg_show_button, NULL);
+	}
+	else
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rg_show_button), TRUE);
+
 	tmp = g_strdup_printf(_("File Info - %s"), g_basename(vte.filename));
 	gtk_window_set_title(GTK_WINDOW(window), tmp);
 	g_free(tmp);
@@ -765,13 +910,22 @@ void vorbis_file_info_box(char *fn)
 	g_free(track_name);
 	g_free(performer);
 	g_free(album_name);
-	g_free(date);
 	g_free(track_number);
 	g_free(genre);
-	g_free(description);
+	g_free(date);
+	g_free(user_comment);
 	g_free(location);
+	g_free(description);
 	g_free(version);
 	g_free(isrc);
 	g_free(organization);
 	g_free(copyright);
+	g_free(rg_track_gain);
+	g_free(rg_album_gain);
+	g_free(rg_track_peak);
+	g_free(rg_album_peak);
+}
+
+static void vorbis_advanced_tag_edit()
+{
 }
