@@ -1,5 +1,5 @@
-/* This program is licensed under the GNU General Public License, version 2,
- * a copy of which is included with this program.
+/* This program is licensed under the GNU Library General Public License, version 2,
+ * a copy of which is included with this program (with filename LICENSE.LGPL).
  *
  * (c) 2000-2001 Michael Smith <msmith@labyrinth.net.au>
  *
@@ -26,6 +26,7 @@ typedef struct {
 	ogg_stream_state	*os;
 
 	vorbis_comment		*vc;
+	vorbis_info          vi;
 
 	vcedit_read_func read;
 	vcedit_write_func write;
@@ -37,6 +38,10 @@ typedef struct {
 	int		mainlen;
 	int		booklen;
 	char 	    *lasterror;
+	char   *vendor;
+	int prevW;
+	int extrapage;
+	int eosin;
 } vcedit_state;
 
 extern vcedit_state *	vcedit_new_state(void);
