@@ -1,17 +1,40 @@
 /*
- * Vorbis xmms module, tested with xmms 0.9.5.1 and xmms CVS source.
- * Report problems to Tony Arcieri, <bascule@inferno.tusculum.edu>
+ * Copyright (C) Tony Arcieri <bascule@inferno.tusculum.edu>
+ * Copyright (C) 2001-2002  Haavard Kvaalen <havardk@xmms.org>
+ *
+ * ReplayGain processing Copyright (C) 2002 Gian-Carlo Pascutto <gcp@sjeng.org>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ *
  */
 
-/* Note that this uses vorbisfile.a, which is not (currently)
-   thread-safe.  Right now, we only make vorbisfile calls from one thread
-   so no problem.  Looks like I better at least add locking to
-   vorbisfile.a ASAP (it won't require any API changes) --Monty 19991112 */
+/*
+ * 2002-01-11 ReplayGain processing added by Gian-Carlo Pascutto <gcp@sjeng.org>
+ */
+
+/*
+ * Note that this uses vorbisfile, which is not (currently)
+ * thread-safe.
+ */
 
 #include "config.h"
 #include <stdio.h>
 #include <fcntl.h>
 #include <stdlib.h>
+#include <math.h>
 #include <string.h>
 #include <pthread.h>
 #include <glib.h>
@@ -38,9 +61,12 @@ static void vorbis_pause(short p);
 static void vorbis_seek(int time);
 static int vorbis_time(void);
 static void vorbis_get_song_info(char *filename, char **title, int *length);
-static gchar *vorbis_generate_title(OggVorbis_File *comment, char *fn);
+static gchar *vorbis_generate_title(OggVorbis_File *vorbisfile, char *fn);
 static void vorbis_aboutbox(void);
 static void vorbis_init(void);
+static long vorbis_process_replaygain(float **pcm, int samples, int ch, char *pcmout, float rg_scale);
+static gboolean vorbis_update_replaygain(float *scale);
+
 
 static size_t ovcb_read(void *ptr, size_t size, size_t nmemb, void *datasource);
 static int ovcb_seek(void *datasource, int64_t offset, int whence);
@@ -163,9 +189,13 @@ static void vorbis_jump_to_time(long time)
 }
 
 #ifdef WORDS_BIGENDIAN
-#define host_is_big_endian() TRUE
+#define XMMS_BIG_ENDIAN TRUE
+#define GET_BYTE1(val) ((val) >> 8)
+#define GET_BYTE2(val) ((val) & 0xff)
 #else
-#define host_is_big_endian() FALSE
+#define XMMS_BIG_ENDIAN FALSE
+#define GET_BYTE1(val) ((val) & 0xff)
+#define GET_BYTE2(val) ((val) >> 8)
 #endif
 
 static void do_seek(void)
@@ -177,11 +207,12 @@ static void do_seek(void)
 	}
 }
 
-static int vorbis_process_data(int last_section)
+static int vorbis_process_data(int last_section, gboolean use_rg, float rg_scale)
 {
 	char pcmout[4096];
-	int bep = host_is_big_endian();
-	int ret;
+	const int bep = XMMS_BIG_ENDIAN;
+	int bytes;
+	float **pcm;
 
 	/*
 	 * A vorbis physical bitstream may consist of many logical
@@ -193,8 +224,16 @@ static int vorbis_process_data(int last_section)
 	int current_section;
 
 	pthread_mutex_lock(&vf_mutex);
-	ret = ov_read(&vf, pcmout, sizeof(pcmout), bep, 2, 1, &current_section);
-	switch (ret)
+	if (use_rg) {
+		bytes = ov_read_float(&vf, &pcm, sizeof(pcmout)/2/channels, &current_section);
+		if (bytes > 0)
+			bytes = vorbis_process_replaygain(pcm, bytes, channels,
+			                                  pcmout, rg_scale);
+	} else {
+		bytes = ov_read(&vf, pcmout, sizeof(pcmout), bep, 2, 1, &current_section);
+	}
+
+	switch (bytes)
 	{
 		case 0:
 			/* EOF */
@@ -203,7 +242,9 @@ static int vorbis_process_data(int last_section)
 			vorbis_ip.output->buffer_free();
 			vorbis_eos = TRUE;
 			return last_section;
-		case -1:
+
+		case OV_HOLE:
+		case OV_EBADLINK:
 			/*
 			 * error in the stream.  Not a problem, just
 			 * reporting it in case we (the app) cares.
@@ -249,13 +290,12 @@ static int vorbis_process_data(int last_section)
 			vorbis_ip.output->flush(ov_time_tell(&vf) * 1000);
 		}
 	}
-	
 	pthread_mutex_unlock(&vf_mutex);
-       
+
 	vorbis_ip.add_vis_pcm(vorbis_ip.output->written_time(), 
-			      FMT_S16_NE, channels, ret, pcmout);
+			      FMT_S16_NE, channels, bytes, pcmout);
      
-	while (vorbis_ip.output->buffer_free() < ret)
+	while (vorbis_ip.output->buffer_free() < bytes)
 	{
 		xmms_usleep(20000);
 		if (!vorbis_playing)
@@ -263,7 +303,7 @@ static int vorbis_process_data(int last_section)
 		if (seekneeded != -1)
 			do_seek();
 	}
-	vorbis_ip.output->write_audio(pcmout, ret);
+	vorbis_ip.output->write_audio(pcmout, bytes);
 
 	return current_section;
 }
@@ -281,6 +321,9 @@ static void *vorbis_play_loop(void *arg)
 	FILE *stream = NULL;
 	void *datasource = NULL;
 
+	gboolean use_rg;
+	float rg_scale;
+
 	memset(&vf, 0, sizeof(vf));
 
 	if (strncasecmp("http://", filename, 7) != 0) {
@@ -304,9 +347,9 @@ static void *vorbis_play_loop(void *arg)
 	 */
 
 	pthread_mutex_lock(&vf_mutex);
-	if (ov_open_callbacks(datasource, &vf, NULL, 0, vorbis_callbacks) < 0) {
-		if (!vorbis_is_streaming)
-			fclose((FILE *)datasource);
+	if (ov_open_callbacks(datasource, &vf, NULL, 0, vorbis_callbacks) < 0)
+	{
+		vorbis_callbacks.close_func(datasource);
 		pthread_mutex_unlock(&vf_mutex);
 		vorbis_eos = TRUE;
 		goto play_cleanup;
@@ -328,8 +371,10 @@ static void *vorbis_play_loop(void *arg)
 	channels = vi->channels;
    
 	title = vorbis_generate_title(&vf, filename);
+	use_rg = vorbis_update_replaygain(&rg_scale);
+
 	vorbis_ip.set_info(title, time, ov_bitrate(&vf, -1), samplerate, channels);
-	if (!vorbis_ip.output->open_audio(FMT_S16_LE, vi->rate, vi->channels))
+	if (!vorbis_ip.output->open_audio(FMT_S16_NE, vi->rate, vi->channels))
 	{
 		output_error = TRUE;
 		pthread_mutex_unlock(&vf_mutex);
@@ -346,20 +391,24 @@ static void *vorbis_play_loop(void *arg)
 	 * using the ov_ interface.
 	 */
 
-	while (vorbis_playing) {
+	while (vorbis_playing)
+	{
 		int current_section;
 
 		if (seekneeded != -1)
 			do_seek();
 
-		if (vorbis_eos) {
+		if (vorbis_eos)
+		{
 			xmms_usleep(20000);
 			continue;
 		}
 			
-		current_section = vorbis_process_data(last_section);
+		current_section = vorbis_process_data(last_section,
+						      use_rg, rg_scale);
 
-		if (current_section != last_section) {
+		if (current_section != last_section)
+		{
 			/*
 			 * set total play time, bitrate, rate, and channels of
 			 * current section
@@ -368,7 +417,8 @@ static void *vorbis_play_loop(void *arg)
 				g_free(title);
 			pthread_mutex_lock(&vf_mutex);
 			title = vorbis_generate_title(&vf, filename);
-			
+			use_rg = vorbis_update_replaygain(&rg_scale);
+
 			if (vorbis_is_streaming)
 				time = -1;
 			else
@@ -385,7 +435,8 @@ static void *vorbis_play_loop(void *arg)
 
 		if (!(vi->bitrate_upper == vi->bitrate_lower == vi->bitrate_nominal) &&
 		     (vorbis_ip.output->output_time() > timercount + 1000 ||
-		      vorbis_ip.output->output_time() < timercount)) {
+		      vorbis_ip.output->output_time() < timercount))
+		{
 			/*
 			 * simple hack to avoid updating too
 			 * often
@@ -514,19 +565,118 @@ static gchar *extname(const char *filename)
 	return ext;
 }
 
+/* Make sure you've locked vf_mutex */
+static gboolean vorbis_update_replaygain(float *scale)
+{
+	vorbis_comment *comment;
+	char *rg_gain = NULL, *rg_peak_str = NULL;
+	float rg_peak;
+
+	if (!vorbis_cfg.use_replaygain && !vorbis_cfg.use_anticlip)
+		return FALSE;
+	if ((comment = ov_comment(&vf, -1)) == NULL)
+		return FALSE;
+
+	*scale = 1.0;
+
+	if (vorbis_cfg.use_replaygain)
+	{
+		if (vorbis_cfg.replaygain_mode == REPLAYGAIN_MODE_ALBUM)
+		{
+			rg_gain = vorbis_comment_query(comment, "replaygain_album_gain", 0);
+			if (!rg_gain)
+				rg_gain = vorbis_comment_query(comment, "rg_audiophile", 0); /* Old */
+		}
+
+		if (!rg_gain)
+			rg_gain = vorbis_comment_query(comment, "replaygain_track_gain", 0);
+		if (!rg_gain)
+			rg_gain = vorbis_comment_query(comment, "rg_radio", 0); /* Old */
+		
+		/* FIXME: Make sure this string is the correct format first? */
+		if (rg_gain)
+			*scale = pow(10., atof(rg_gain)/20);
+	}
+	
+	if (vorbis_cfg.use_anticlip)
+	{
+		if (vorbis_cfg.replaygain_mode == REPLAYGAIN_MODE_ALBUM)
+			rg_peak_str = vorbis_comment_query(comment, "replaygain_album_peak", 0);
+
+		if (!rg_peak_str)
+			rg_peak_str = vorbis_comment_query(comment, "replaygain_track_peak", 0);
+		if (!rg_peak_str)
+			rg_peak_str = vorbis_comment_query(comment, "rg_peak", 0); /* Old */
+
+		if (rg_peak_str)
+			rg_peak = atof(rg_peak_str);
+		else
+			rg_peak = 1;
+
+		if (*scale * rg_peak > 1.0)
+			*scale = 1.0 / rg_peak;
+	}
+	
+	if (*scale != 1.0 || vorbis_cfg.use_booster) {
+		/* safety */
+		if (*scale > 15.0)
+			*scale = 15.0;
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static long vorbis_process_replaygain(float **pcm, int samples, int ch, char *pcmout, float rg_scale)
+{
+	int i, j;
+	/* ReplayGain processing */
+	for (i = 0; i < samples; i++)
+		for (j = 0; j < ch; j++)
+		{
+			float sample = pcm[j][i] * rg_scale;
+			int value;
+				
+			if (vorbis_cfg.use_booster)
+			{
+				sample *= 2;
+				
+				/* hard 6dB limiting */
+				if (sample < -0.5)
+					sample = tanh((sample + 0.5) / 0.5) * 0.5 - 0.5;
+				else if (sample > 0.5)
+					sample = tanh((sample - 0.5) / 0.5) * 0.5 + 0.5;
+			}
+				
+			value = sample * 32767;
+			if (value > 32767)
+				value = 32767;
+			else if (value < -32767)
+				value = -32767;
+			
+			*pcmout++ = GET_BYTE1(value);
+			*pcmout++ = GET_BYTE2(value);
+		}
+
+	return 2 * ch * samples;
+}			
 
 static gchar *vorbis_generate_title(OggVorbis_File *vorbisfile, char *fn)
 {
 	/* Caller should hold vf_mutex */
-	gchar *displaytitle = NULL, *tmp;
+	gchar *displaytitle = NULL, *tmp, *path, *temp;
 	vorbis_comment *comment;
 	TitleInput *input;
 
 	XMMS_NEW_TITLEINPUT(input);
-	
+	path = g_strdup(fn);
+	temp = strrchr(path, '/');
+	if (temp)
+		*temp = '\0';
 	input->file_name = g_basename(fn);
 	input->file_ext = extname(fn);
-	input->file_path = fn;
+	input->file_path = g_strdup_printf("%s/", path);
 
 	if ((comment = ov_comment(vorbisfile, -1)) != NULL)
 	{
@@ -537,8 +687,8 @@ static gchar *vorbis_generate_title(OggVorbis_File *vorbisfile, char *fn)
 			input->track_number = atoi(tmp);
 		input->date = convert_from_utf8(vorbis_comment_query(comment, "date", 0));
 		input->genre = convert_from_utf8(vorbis_comment_query(comment, "genre", 0));
-		/* "" = user comment */
-		input->comment = convert_from_utf8(vorbis_comment_query(comment, "", 0));
+		/* "comment" = user comment, not "" */
+		input->comment = convert_from_utf8(vorbis_comment_query(comment, "comment", 0));
 	}
 
 	displaytitle = xmms_get_titlestring(vorbis_cfg.tag_override ?
@@ -551,8 +701,10 @@ static gchar *vorbis_generate_title(OggVorbis_File *vorbisfile, char *fn)
 	g_free(input->genre);
 	g_free(input->comment);
 	g_free(input);
+	g_free(path);
 
-	if (!displaytitle) {
+	if (!displaytitle)
+	{
 		if (!vorbis_is_streaming)
 			displaytitle = g_strdup_printf("%s", g_basename(fn));
 		else
@@ -571,7 +723,11 @@ static void vorbis_aboutbox()
 
 	about_window = xmms_show_message(
 		_("About Ogg Vorbis Plugin"),
-		_("Ogg Vorbis Plugin by Xiphophorus\n\n"
+		/*
+		 * I18N: UTF-8 Translation: "Haavard Kvaalen" ->
+		 * "H\303\245vard Kv\303\245len"
+		 */
+		_("Ogg Vorbis Plugin by the Xiph.org Foundation\n\n"
 		  "Original code by\n"
 		  "Tony Arcieri <bascule@inferno.tusculum.edu>\n"
 		  "Contributions from\n"
@@ -580,8 +736,9 @@ static void vorbis_aboutbox()
 		  "Michael Smith <msmith@labyrinth.edu.au>\n"
 		  "Jack Moffitt <jack@icecast.org>\n"
 		  "Jorn Baayen <jorn@nl.linux.org>\n"
-		  "Hvard Kvlen <havardk@xmms.org>\n\n"
-		  "Visit Xiphophorus at http://www.xiph.org/\n"),
+		  "Haavard Kvaalen <havardk@xmms.org>\n"
+		  "Gian-Carlo Pascutto <gcp@sjeng.org>\n\n"
+		  "Visit the Xiph.org Foundation at http://www.xiph.org/\n"),
 		_("Ok"), FALSE, NULL, NULL);
 	gtk_signal_connect(GTK_OBJECT(about_window), "destroy",
 			   GTK_SIGNAL_FUNC(gtk_widget_destroyed),
@@ -592,6 +749,7 @@ static void vorbis_init(void)
 {
 	ConfigFile *cfg;
 
+	memset(&vorbis_cfg, 0, sizeof(vorbis_config_t));
 	vorbis_cfg.http_buffer_size = 128;
 	vorbis_cfg.http_prebuffer = 25;
 	vorbis_cfg.proxy_port = 8080;
@@ -600,24 +758,43 @@ static void vorbis_init(void)
 	vorbis_cfg.proxy_pass = NULL;
 	vorbis_cfg.tag_override = FALSE;
 	vorbis_cfg.tag_format = NULL;
+	vorbis_cfg.use_anticlip = FALSE;
+	vorbis_cfg.use_replaygain = FALSE;
+	vorbis_cfg.replaygain_mode = REPLAYGAIN_MODE_TRACK;
+	vorbis_cfg.use_booster = FALSE;
 
 	cfg = xmms_cfg_open_default_file();
-	xmms_cfg_read_int(cfg, "vorbis", "http_buffer_size", &vorbis_cfg.http_buffer_size);
-	xmms_cfg_read_int(cfg, "vorbis", "http_prebuffer", &vorbis_cfg.http_prebuffer);
-	xmms_cfg_read_boolean(cfg, "vorbis", "save_http_stream", &vorbis_cfg.save_http_stream);
-	if (!xmms_cfg_read_string(cfg, "vorbis", "save_http_path", &vorbis_cfg.save_http_path))
+	xmms_cfg_read_int(cfg, "vorbis", "http_buffer_size",
+			  &vorbis_cfg.http_buffer_size);
+	xmms_cfg_read_int(cfg, "vorbis", "http_prebuffer",
+			  &vorbis_cfg.http_prebuffer);
+	xmms_cfg_read_boolean(cfg, "vorbis", "save_http_stream",
+			      &vorbis_cfg.save_http_stream);
+	if (!xmms_cfg_read_string(cfg, "vorbis", "save_http_path",
+				  &vorbis_cfg.save_http_path))
 		vorbis_cfg.save_http_path = g_strdup(g_get_home_dir());
 
 	xmms_cfg_read_boolean(cfg, "vorbis", "use_proxy", &vorbis_cfg.use_proxy);
-	if (!xmms_cfg_read_string(cfg, "vorbis", "proxy_host", &vorbis_cfg.proxy_host))
+	if (!xmms_cfg_read_string(cfg, "vorbis", "proxy_host",
+				  &vorbis_cfg.proxy_host))
 		vorbis_cfg.proxy_host = g_strdup("localhost");
 	xmms_cfg_read_int(cfg, "vorbis", "proxy_port", &vorbis_cfg.proxy_port);
-	xmms_cfg_read_boolean(cfg, "vorbis", "proxy_use_auth", &vorbis_cfg.proxy_use_auth);
-	xmms_cfg_read_string(cfg, "vorbis", "proxy_user", &vorbis_cfg.proxy_user);
-	xmms_cfg_read_string(cfg, "vorbis", "proxy_pass", &vorbis_cfg.proxy_pass);
-	xmms_cfg_read_boolean(cfg, "vorbis", "tag_override", &vorbis_cfg.tag_override);
-	if (!xmms_cfg_read_string(cfg, "vorbis", "tag_format", &vorbis_cfg.tag_format))
+	xmms_cfg_read_boolean(cfg, "vorbis", "proxy_use_auth",
+			      &vorbis_cfg.proxy_use_auth);
+	xmms_cfg_read_string(cfg, "vorbis", "proxy_user",
+			     &vorbis_cfg.proxy_user);
+	xmms_cfg_read_string(cfg, "vorbis", "proxy_pass",
+			     &vorbis_cfg.proxy_pass);
+	xmms_cfg_read_boolean(cfg, "vorbis", "tag_override",
+			      &vorbis_cfg.tag_override);
+	if (!xmms_cfg_read_string(cfg, "vorbis", "tag_format",
+				  &vorbis_cfg.tag_format))
 		vorbis_cfg.tag_format = g_strdup("%p - %t");
+	xmms_cfg_read_boolean(cfg, "vorbis", "use_anticlip", &vorbis_cfg.use_anticlip);
+	xmms_cfg_read_boolean(cfg, "vorbis", "use_replaygain", &vorbis_cfg.use_replaygain);
+	xmms_cfg_read_int(cfg, "vorbis", "replaygain_mode", &vorbis_cfg.replaygain_mode);
+	xmms_cfg_read_boolean(cfg, "vorbis", "use_booster", &vorbis_cfg.use_booster);
+	xmms_cfg_free(cfg);
 }
 
 static size_t ovcb_read(void *ptr, size_t size, size_t nmemb, void *datasource)
