@@ -36,7 +36,6 @@ static char *buffer_read = NULL;
 static char *buffer_write = NULL;
 static int buffer_len = 0;
 static pthread_mutex_t buflock = PTHREAD_MUTEX_INITIALIZER;
-static struct timespec min_wait;
 
 static gint time_offset = 0;
 static gint byte_offset = 0;
@@ -44,6 +43,8 @@ static gint byte_offset = 0;
 static gboolean paused = FALSE;
 static gboolean playing = FALSE;
 static gboolean aopen = FALSE;
+static gboolean prebuffer, remove_prebuffer;
+static int prebuffer_size;
 static volatile gint flush = -1;
 
 static gint audiofd = -2;
@@ -161,8 +162,8 @@ static void audio_yield(void)
 
 	while (playing && (flush == -1) && ((info.play.eof + 10) < audio_scount))
 	{
-		nanosleep(&min_wait, NULL);
-		ioctl(audiofd, AUDIO_GETINFO, &info);
+	        xmms_usleep(10000);
+	        ioctl(audiofd, AUDIO_GETINFO, &info);
 	}
 }
 
@@ -176,6 +177,7 @@ static void audio_play(char *bufferP, int length)
 		if (written > 0)
 		{
 			length -= written;
+			bufferP += written;
 			if (write(audiofd, NULL, 0) == 0)
 			{
 				audio_scount++;
@@ -184,7 +186,7 @@ static void audio_play(char *bufferP, int length)
 		}
 		else
 		{
-			nanosleep(&min_wait, NULL);
+		        xmms_usleep(10000);
 		}
 	}
 }
@@ -220,12 +222,24 @@ gint abuffer_used(void)
 }
 
 /*
- * This is supposed to allow reclaim of unplayed music;
- * Requires mucking about with the buffer
+ * Doesn't free the buffer. Just returns the number of bytes
+ * in the buffer available to write.
  */
 gint abuffer_free(void)
 {
 	int retval;
+
+	/*
+	 * Two calls to this function without a write in between,
+	 * means we are at the end of the stream so stop prebuffering.
+	 */
+	if (remove_prebuffer && prebuffer)
+	{
+		prebuffer = FALSE;
+		remove_prebuffer = FALSE;
+	}
+	if (prebuffer)
+		remove_prebuffer = TRUE;
 	/* Find the amount of buffer free (circular buffer) */
 	pthread_mutex_lock(&buflock);
 	if(buffer_write >= buffer_read)
@@ -359,6 +373,8 @@ void abuffer_write(void *ptr, gint length)
 	AFormat new_format;
 	gint new_frequency, new_channels;
 	EffectPlugin *ep;
+
+	remove_prebuffer = FALSE;
 	
 	new_format = input_format;
 	new_frequency = stream_attrs.sample_rate;
@@ -384,7 +400,7 @@ void abuffer_write(void *ptr, gint length)
 /*  		close(fd); */
 /*  		fd = open(device_name,O_WRONLY); */
 /*  		oss_set_audio_params(); */
-		//FIXME: This is leaking memory
+		/* FIXME: This is leaking memory */
 		abuffer_open(new_format, new_frequency, new_channels);
 		
 	}
@@ -474,8 +490,7 @@ gint abuffer_startup(void)
 	/* Open audio device if it's not already open */
 	if (audiofd < 0)
 	{
-		if ((audiofd = open(get_audiodev(),
-				    (O_WRONLY | O_NONBLOCK))) == -1)
+		if ((audiofd = open(get_audiodev(), O_WRONLY )) == -1)
 		{
 			perror("xmms Solaris Output plugin");
 			return 1;
@@ -504,6 +519,9 @@ gint abuffer_startup(void)
 	/* We're open */
 	aopen = TRUE;
 
+	prebuffer = TRUE;
+	remove_prebuffer = FALSE;
+
 	/* We're at the beginning */
 	time_offset = 0;
 	byte_offset = 0;
@@ -531,21 +549,16 @@ void abuffer_shutdown(void)
 	/* Free the buffer */
 	g_free(buffer_ptr);
 	buffer_ptr = buffer_end = buffer_read = buffer_write = NULL;
-
-	/* Exit the play loop thread */
-	pthread_exit(NULL);
 }
 
 void * abuffer_loop(void *arg)
 {
 	gint length, bdiff = paused;
-	gint used;
 
-	/* FIXME: I don't think prebuffering works */
 	while (playing)
 	{
-		/* Find out how much buffer is used */
-		used = abuffer_used();
+		if (abuffer_used() > prebuffer_size)
+			prebuffer = FALSE;
 
 		/* If pause has changed status, go ahead and do it */
 		if (adinfo.play.pause ^ paused)
@@ -566,10 +579,10 @@ void * abuffer_loop(void *arg)
 		}
 
 		/* If we're not paused and have data, send it */
-		if ((used > 0) && !paused)
+		if (abuffer_used() > 0 && !paused && !prebuffer)
 		{
 			/* we have something in the buffer and we're not paused */
-			length = MIN(SUN_AUDIO_CACHE, used); 
+			length = MIN(SUN_AUDIO_CACHE, abuffer_used()); 
 			bdiff = buffer_end - buffer_read;
 
 			/* Write out the buffer */
@@ -588,11 +601,6 @@ void * abuffer_loop(void *arg)
 				buffer_read += length;
 				pthread_mutex_unlock(&buflock);
 			}
-
-			/* Sleep to the desired amount before resend */
-			if (length > SUN_AUDIO_CACHE)
-				nanosleep(&min_wait, NULL);
-
 		}
 		else if (flush != -1)
 		{
@@ -603,7 +611,13 @@ void * abuffer_loop(void *arg)
 			ioctl(audiofd, AUDIO_SETINFO, &info);
             
 			/* Flush out the queue */
+                        /* Yes this is voodoo magic. On my box, this works.
+                           But, if I remove the calls to xmms_usleep(), the
+                           I_FLUSH ioctl puts the audio into a screwy state
+                           about a quarter of the time. */
+			xmms_usleep(10000);
 			ioctl(audiofd, I_FLUSH, FLUSHW);
+			xmms_usleep(10000);
 
 			/* Make the structure harmless */
 			AUDIO_INITINFO(&info);
@@ -807,12 +821,9 @@ gint abuffer_open(AFormat fmt, gint rate, gint nch)
 
 	/* Define the length for the buffer */
 	buffer_len = (sun_cfg.buffer_size * stream_attrs.bps) / 1000;
-
-	/* Define the minimum time we wait after filling the buffer */
-	min_wait.tv_sec = 0;
-	min_wait.tv_nsec = (((100.0 - (float) sun_cfg.prebuffer)
-			     * SUN_AUDIO_CACHE * 10000000.0)
-			    / (float) stream_attrs.bps);
+	prebuffer_size = (buffer_len * sun_cfg.prebuffer) / 100;
+	if (buffer_len - prebuffer_size < 4096)
+		prebuffer_size = buffer_len - 4096;
 
 	buffer_ptr = g_malloc0(buffer_len);
 	buffer_write = buffer_ptr;
@@ -849,7 +860,6 @@ gint abuffer_open(AFormat fmt, gint rate, gint nch)
 
 /*
  * xmms expects 0<=volume<=100; Solaris audio is 0<=vol<=255
- * Have to translate using hideous amounts of casts
  */
 
 /*
