@@ -1,7 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-2002  Peter Alm, Mikael Alm, Olle Hallnas,
+ *  Copyright (C) 1998-2003  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999-2002  Haavard Kvaalen
+ *  Copyright (C) 1999-2003  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,16 +38,33 @@
 #endif
 
 
+static int read_all(int fd, void *buf, size_t count)
+{
+	size_t left = count;
+	do {
+		int r = read(fd, buf, left);
+		if (r < 0)
+			return -1;
+		left -= r;
+		buf = (char *)buf + r;
+	} while (left > 0);
+	return count;
+}
+
 static gpointer remote_read_packet(gint fd, ServerPktHeader * pkt_hdr)
 {
 	gpointer data = NULL;
 
-	if (read(fd, pkt_hdr, sizeof (ServerPktHeader)) == sizeof (ServerPktHeader))
+	if (read_all(fd, pkt_hdr, sizeof (ServerPktHeader)) == sizeof (ServerPktHeader))
 	{
 		if (pkt_hdr->data_length)
 		{
 			data = g_malloc0(pkt_hdr->data_length);
-			read(fd, data, pkt_hdr->data_length);
+			if (read_all(fd, data, pkt_hdr->data_length) < 0)
+			{
+				g_free(data);
+				data = NULL;
+			}
 		}
 	}
 	return data;
@@ -64,6 +81,25 @@ static void remote_read_ack(gint fd)
 
 }
 
+static int write_all(int fd, const void *buf, size_t count)
+{
+	size_t left = count;
+	/* FIXME: This can take forever */
+	do {
+		int written = write(fd, buf, left);
+		if (written < 0)
+		{
+			g_warning("remote_send_packet(): "
+				  "Failed to send data to xmms: %s",
+				  strerror(errno));
+			return -1;
+		}
+		left -= written;
+		buf = (char *)buf + written;
+	} while (left > 0);
+	return count;
+}
+
 static void remote_send_packet(gint fd, guint32 command, gpointer data, guint32 data_length)
 {
 	ClientPktHeader pkt_hdr;
@@ -71,9 +107,10 @@ static void remote_send_packet(gint fd, guint32 command, gpointer data, guint32
 	pkt_hdr.version = XMMS_PROTOCOL_VERSION;
 	pkt_hdr.command = command;
 	pkt_hdr.data_length = data_length;
-	write(fd, &pkt_hdr, sizeof (ClientPktHeader));
+	if (write_all(fd, &pkt_hdr, sizeof (ClientPktHeader)) < 0)
+		return;
 	if (data_length && data)
-		write(fd, data, data_length);
+		write_all(fd, data, data_length);
 }
 
 static void remote_send_guint32(gint session, guint32 cmd, guint32 val)
@@ -254,12 +291,15 @@ void xmms_remote_playlist(gint session, gchar ** list, gint num, gboolean enqueu
 	gint data_length;
 	guint32 len;
 
-	if ((fd = xmms_connect_to_session(session)) == -1)
-		return;
-
+	g_return_if_fail(list != NULL);
+	g_return_if_fail(num > 0);
+	
 	if (!enqueue)
 		xmms_remote_playlist_clear(session);
 
+	if ((fd = xmms_connect_to_session(session)) == -1)
+		return;
+
 	for (i = 0, data_length = 0; i < num; i++)
 		data_length += (((strlen(list[i]) + 1) + 3) / 4) * 4 + 4;
 	if (data_length)
@@ -292,6 +332,8 @@ gint xmms_remote_get_version(gint session)
 
 void xmms_remote_play_files(gint session, GList * list)
 {
+	g_return_if_fail(list != NULL);
+
 	xmms_remote_playlist_clear(session);
 	xmms_remote_add_files(session, list);
 	xmms_remote_play(session);
@@ -303,6 +345,8 @@ void xmms_remote_playlist_add(gint session, GList * list)
 	GList *node;
 	gint i, num;
 
+	g_return_if_fail(list != NULL);
+	
 	num = g_list_length(list);
 	str_list = g_malloc0(num * sizeof (gchar *));
 	for (i = 0, node = list; i < num && node; i++, node = g_list_next(node))
@@ -385,6 +429,7 @@ void xmms_remote_get_volume(gint session, gint * vl, gint * vr)
 
 	if ((fd = xmms_connect_to_session(session)) == -1)
 		return;
+
 	remote_send_packet(fd, CMD_GET_VOLUME, NULL, 0);
 	data = remote_read_packet(fd, &pkt_hdr);
 	if (data)
@@ -615,6 +660,7 @@ void xmms_remote_playlist_next(gint session)
 
 void xmms_remote_playlist_add_url_string(gint session, gchar * string)
 {
+	g_return_if_fail(string != NULL);
 	remote_send_string(session, CMD_PLAYLIST_ADD_URL_STRING, string);
 }
 
@@ -725,10 +771,12 @@ void xmms_remote_set_eq(gint session, gfloat preamp, gfloat *bands)
 	gint fd, i;
 	gfloat data[11];
 
+	g_return_if_fail(bands != NULL);
+
 	if ((fd = xmms_connect_to_session(session)) == -1)
 		return;
 	data[0] = preamp;
-	for(i = 0; i < 10; i++)
+	for (i = 0; i < 10; i++)
 		data[i + 1] = bands[i];
 	remote_send_packet(fd, CMD_SET_EQ, data, sizeof(data));
 	remote_read_ack(fd);
