@@ -1,7 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-2002  Peter Alm, Mikael Alm, Olle Hallnas,
+ *  Copyright (C) 1998-2003  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 2000-2002  Haavard Kvaalen
+ *  Copyright (C) 2000-2003  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public Licensse as published by
@@ -24,16 +24,17 @@
  * The different sections are kept in alphabetical order
  */
 
-static const gchar *credit_text[] =
+static const char *credit_text[] =
 {N_("Main Programming:") ,
 	N_("Peter Alm"),
 	NULL,
  N_("Additional Programming:"),
-/* for languages that can't display 'a ring' () replace it with "aa" */
-	N_("Hvard Kvlen"),
+	/* I18N: UTF-8 Translation: "H\303\245vard Kv\303\245len" */
+	N_("Haavard Kvaalen"),
 	N_("Derrik Pates"),
 	NULL,
  N_("With Additional Help:"),
+	N_("Tony Arcieri"),
 	N_("Sean Atkinson"),
 	N_("Jorn Baayen"),
 	N_("James M. Cape"),
@@ -45,11 +46,12 @@ static const gchar *credit_text[] =
 	N_("Logan Hanks"),
 	N_("Eric L. Hernes (FreeBSD patches)"),
 	N_("Ville Herva"),
-        N_("Michael Hipp and others (MPG123 engine)"),
-/* for languages that can't display 'a diaeresis' () replace it with "ae" */
-	N_("Olle Hllns (compiling fixes)"),
-        N_("David Jacoby"),
-	N_("Osamu Kayasono (3DNow!)"),
+	N_("higway (MMX)"),
+	N_("Michael Hipp and others (MPG123 engine)"),
+	/* I18N: UTF-8 translation: "Olle H\303\244lln\303\244s ..." */
+	N_("Olle Hallnas (compiling fixes)"),
+	N_("David Jacoby"),
+	N_("Osamu Kayasono (3DNow!/MMX)"),
 	N_("Lyle B Kempler"),
 	N_("J. Nick Koston (MikMod plugin)"),
 	N_("Aaron Lehmann"),
@@ -57,13 +59,16 @@ static const gchar *credit_text[] =
 	N_("Eric Lindvall"),
 	N_("Colin Marquardt"),
 	N_("Willem Monsuwe"),
+	N_("Gian-Carlo Pascutto"),
 	N_("John Riddoch (Solaris plugin)"),
+	N_("Josip Rodin"),
 	N_("Pablo Saratxaga (i18n)"),
 	N_("Carl van Schaik (pro logic plugin)"),
-/* for languages that can't display 'o diaeresis' () replace it with "oe" */
-	N_("Jrg Schuler"),
+	/* I18N: UTF-8 translation: "J\303\266rg Schuler" */
+	N_("Joerg Schuler"),
 	N_("Charles Sielski (irman plugin)"),
 	N_("Espen Skoglund"),
+	N_("Matthieu Sozeau (ALSA plugin)"),
 	N_("Kimura Takuhiro (3DNow!)"),
 	N_("Zinx Verituse"),
 	N_("Ryan Weaver (RPMs among other things)"),
@@ -71,44 +76,52 @@ static const gchar *credit_text[] =
 	N_("Dave Yearke"),
 	N_("Stephan K. Zitz"),
 	NULL,
+ N_("Default skin:"),
+ 	N_("Leonard \"Blayde\" Tan"),
+	N_("Robin Sylvestre (Equalizer and Playlist)"),
+	N_("Thomas Nilsson (New titles, and cleanups)"),
+	NULL,
  N_("Homepage and Graphics:"),
 	N_("Thomas Nilsson"),
 	NULL,
  N_("Support and Docs:"),
-/* for languages that can't display 'a diaeresis' () replace it with "ae" */
-	N_("Olle Hllns"),
+	/* I18N: UTF-8 translation: "Olle H\303\244lln\303\244s" */
+	N_("Olle Hallnas"),
 	NULL, NULL};
 
 static const char *translators[] =
 {
 	N_("Afrikaans:"),
-	/* for languages that can't display 'e acute' () replace it with "e" */
-	N_("Schalk W. Cronj "), NULL,
+	/* I18N: UTF-8 translation: "Schalk W. Cronj\303\251" */
+	N_("Schalk W. Cronje"), NULL,
 	N_("Azerbaijani:"), N_("Vasif Ismailoglu"), NULL,
-	N_("Basque:"), N_("Iigo Salvador Azurmendi"), NULL,
+	N_("Basque:"),
+	/* I18N: UTF-8 translation: "I\303\261igo Salvador Azurmendi" */
+	N_("Inigo Salvador Azurmendi"), NULL,
+	N_("Belarusian:"), N_("Smaliakou Zmicer"), NULL,
 	N_("Brazilian Portuguese:"), N_("Juan Carlos Castro y Castro"),	NULL,
 	N_("Bulgarian:"), N_("Yovko D. Lambrev"), NULL,
-	N_("Catalan:"), N_("Quico Llach"), NULL,
+	N_("Catalan:"), N_("Quico Llach"), N_("Jordi Mallach"), NULL,
 	N_("Chinese:"), N_("Chun-Chung Chen"), N_("Jouston Huang"),
 	N_("Andrew Lee"), N_("Chih-Wei Huang"), N_("Danny Zeng"), NULL,
 	N_("Croatian:"), N_("Vlatko Kosturjak"), N_("Vladimir Vuksan"), NULL,
 	N_("Czech:"),
-	/* for languages that can't display 'i acute' () replace it with "i" */
-	N_("Vladimr Marek"),
+	/* I18N: UTF-8 translation: "Vladim\303\255r Marek" */
+	N_("Vladimir Marek"),
 	N_("Radek Vybiral"), NULL,
 	N_("Danish:"), N_("Nikolaj Berg Amondsen"), N_("Troels Liebe Bentsen"),
 	N_("Kenneth Christiansen"), N_("Keld Simonsen"), NULL,
 	N_("Dutch:"), N_("Jorn Baayen"), N_("Wilmer van der Gaast"),
 	N_("Tom Laermans"), NULL,
 	N_("Esperanto:"), N_("D. Dale Gulledge"), NULL,
-	N_("German:"), N_("Colin Marquardt"), N_("Stefan Siegel"), NULL,
-	N_("Greek:"), N_("Kyritsis Athanasios"), NULL,
 	N_("French:"), N_("Arnaud Boissinot"), N_("Eric Fernandez-Bellot"), NULL,
 	N_("Galician:"),
-	/* for languages that can't display 'i acute' () replace it with "i" */
-	N_("Alberto Garca"),
-	/* for languages that can't display 'a acute' () replace it with "a" */
-	N_("David Fernndez Vaamonde"), NULL,
+	/* I18N: UTF-8 translation: "Alberto Garc\303\255a" */
+	N_("Alberto Garcia"),
+	/* I18N: UTF-8 translation: "David Fern\303\241ndez Vaamonde" */
+	N_("David Fernandez Vaamonde"), NULL,
+	N_("German:"), N_("Colin Marquardt"), N_("Stefan Siegel"), NULL,
+	N_("Greek:"), N_("Kyritsis Athanasios"), NULL,
 	N_("Hungarian:"), N_("Arpad Biro"), NULL,
 	N_("Indonesian:"), N_("Budi Rachmanto"), NULL,
 	N_("Irish:"), N_("Alastair McKinstry"), NULL,
@@ -118,43 +131,39 @@ static const char *translators[] =
 	N_("Byeong-Chan Kim"), N_("Man-Yong Lee"), NULL,
 	N_("Lithuanian:"), N_("Gediminas Paulauskas"), NULL,
 	N_("Latvian:"),
-	/* I18N: The "n" should be replaced with 'n cedilla' () if it
-           can be displayed */
-	/* I18N: The last "s" should be replaced with 's caron' () if
-           it can be displayed */
+	/* I18N: UTF-8 translation: "Juris Kudi\305\206\305\241" */
 	N_("Juris Kudins"),
 	N_("Vitauts Stochka"), NULL,
 	N_("Norwegian:"),
-	/* for languages that can't display 'o slash' () replace it with "oe" */
-	N_("Andreas Bergstrm"),
-	N_("Terje Bjerkelia"), N_("Hvard Kvlen"), N_("Roy-Magne Mo"),
+	/* I18N: UTF-8 translation: "Andreas Bergstr\303\270m" */
+	N_("Andreas Bergstrom"),
+	N_("Terje Bjerkelia"), N_("Haavard Kvaalen"), N_("Roy-Magne Mo"),
 	N_("Espen Skoglund"), NULL,
 	N_("Polish:"), N_("Grzegorz Kowal"), NULL,
 	N_("Portuguese:"), N_("Jorge Costa"), NULL,
 	N_("Romanian:"), N_("Florin Grad"),
-	/* I18N: The "s" should be replaced with 's cedilla' () if it
-           can be displayed */
+	/* I18N: UTF-8 translation: "Mi\305\237u Moldovan" */
 	N_("Misu Moldovan"), NULL,
 	N_("Russian:"), N_("Valek Filippov"), N_("Alexandr P. Kovalenko"),
 	N_("Maxim Koshelev"), N_("Aleksey Smirnov"), NULL,
 	N_("Serbian:"), N_("Tomislav Jankovic"), NULL,
-	N_("Slovak:"), N_("Pavol Cvengros"),
+	N_("Slovak:"), N_("Pavol Cvengros"), NULL,
+	N_("Slovenian:"), N_("Tadej Panjtar"),
 	N_("Tomas Hornocek"), N_("Jan Matis"), NULL,
 	N_("Spanish:"), N_("Fabian Mandelbaum"), N_("Jordi Mallach"),
-	/* for languages that can't display 'i acute' () replace it with "i" */
-	N_("Juan Manuel Garca Molina"), NULL,
-	N_("Swedish:"), N_("David Hedbor"), N_("Olle Hllns"),
+	/* I18N: UTF-8 translation: "Juan Manuel Garc\303\255a Molina" */
+	N_("Juan Manuel Garcia Molina"), NULL,
+	N_("Swedish:"), N_("David Hedbor"), N_("Olle Hallnas"),
 	N_("Thomas Nilsson"),	N_("Christian Rose"), N_("Fuad Sabanovic"), NULL,
-	N_("Tajik:"), N_("Trinh Minh Thanh"), NULL,
+	N_("Tajik:"), N_("Roger Kovacs"), N_("Dilshod Marupov"), NULL,
 	N_("Thai:"), N_("Supphachoke Suntiwichaya"), NULL, 
 	N_("Turkish:"), N_("Nazmi Savga"),
-	/* I18N: The "i" should be replaced with 'dotless i' () if it
-           can be displayed */
-	/* for languages that can't display 'O diaeresis' () replace it with "O" */
-	N_("mer Fadil Usta"), NULL,
+	/* I18N: UTF-8 translation: "\303\226mer Fad\304\261l Usta" */
+	N_("Omer Fadil Usta"), NULL,
 	N_("Ukrainian:"), N_("Dmytro Koval'ov"), NULL,
-	N_("Vietnamese:"), N_("Roger Kovacs"), N_("Dilshod Marupov"), NULL,
+	N_("Vietnamese:"), N_("Trinh Minh Thanh"), NULL,
 	N_("Walloon:"), N_("Lucyin Mahin"), N_("Pablo Saratxaga"), NULL,
+	N_("Welsh:"), N_("Rhoslyn Prys"), NULL,
 	NULL
 };
 
@@ -256,7 +265,8 @@ void show_about_window(void)
 	
 	gtk_box_pack_start(GTK_BOX(about_vbox), label, FALSE, FALSE, 0);
 	
-	label = gtk_label_new(_("Copyright (C) 1997-2002 4front Technologies"));
+	label = gtk_label_new(_("Copyright (C) 1997-2003 4front Technologies "
+				"and The XMMS Team"));
 	gtk_box_pack_start(GTK_BOX(about_vbox), label, FALSE, FALSE, 0);
 	
 	about_notebook = gtk_notebook_new();
