@@ -331,9 +331,17 @@ void equalizerwin_press(GtkWidget * widget, GdkEventButton * event, gpointer cal
 	    ((cfg.easy_move || cfg.equalizer_shaded || event->y < 14) &&
 	     !inside_sensitive_widgets(event->x, event->y)))
 	{
-
-		equalizerwin_raise();
-		dock_move_press(dock_window_list, equalizerwin, event, FALSE);
+		if (0 && hint_move_resize_available())
+		{
+			hint_move_resize(equalizerwin, event->x_root,
+					 event->y_root, TRUE);
+			grab = FALSE;
+		}
+		else
+		{
+			equalizerwin_raise();
+			dock_move_press(dock_window_list, equalizerwin, event, FALSE);
+		}
 	}
 	else if (event->button == 1 && event->type == GDK_2BUTTON_PRESS && event->y < 14)
 	{
@@ -1453,6 +1461,7 @@ void equalizerwin_load_auto_preset(gchar * filename)
 	}
 
 	directory = g_dirname(filename);
+	g_free(presetfilename);
 	presetfilename = g_strdup_printf("%s/%s", directory, cfg.eqpreset_default_file);
 	g_free(directory);
 
