@@ -1,7 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-2001  Peter Alm, Mikael Alm, Olle Hallnas,
+ *  Copyright (C) 1998-2003  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999-2001  Haavard Kvaalen
+ *  Copyright (C) 1999-2003  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,8 +35,8 @@ void playlist_delete(gboolean crop);
 #define playlist_add(filename) playlist_ins(filename, -1)
 void playlist_ins(gchar * filename, glong pos);
 /*  void playlist_add_dir(gchar * dir); */
-#define playlist_add_dir(directory) playlist_ins_dir(directory, -1)
-guint playlist_ins_dir(gchar * dir, glong pos);
+#define playlist_add_dir(directory) playlist_ins_dir(directory, -1, TRUE)
+guint playlist_ins_dir(char *dir, long pos, gboolean background);
 /*  void playlist_add_url_string(gchar * string); */
 #define playlist_add_url_string(string) playlist_ins_url_string(string, -1)
 guint playlist_ins_url_string(gchar * string, glong pos);
@@ -45,19 +45,21 @@ void playlist_set_info(gchar * title, gint length, gint rate, gint freq, gint nc
 void playlist_check_pos_current(void);
 void playlist_next(void);
 void playlist_prev(void);
+void playlist_queue(void);
+void playlist_queue_position(gint pos);
+gboolean playlist_is_position_queued(int pos);
+void playlist_clear_queue(void);
+int playlist_get_queue_position(PlaylistEntry *entry);
 void playlist_eof_reached(void);
 void playlist_set_position(gint pos);
 gint get_playlist_length(void);
-gint __get_playlist_length(void);
 gint get_playlist_position(void);
 gint __get_playlist_position(void);
 gchar *playlist_get_info_text(void);
 int playlist_get_current_length(void);
-gboolean playlist_save(gchar * filename);
+gboolean playlist_save(char *filename, gboolean is_pls);
 gboolean playlist_load(gchar * filename);
-guint playlist_load_ins(gchar * filename, glong pos);
 GList *get_playlist(void);
-void playlist_generate_shuffle_list(void);
 void playlist_start_get_info_thread(void);
 void playlist_stop_get_info_thread();
 void playlist_start_get_info_scan(void);
@@ -79,7 +81,9 @@ void playlist_delete_filenames(GList *filenames);
 gchar* playlist_get_filename(gint pos);
 gchar* playlist_get_songtitle(gint pos);
 gint playlist_get_songtime(gint pos);
+GList * playlist_get_selected(void);
 GList * playlist_get_selected_list(void);
+int playlist_get_num_selected(void);
 void playlist_get_total_time(gulong *total_time, gulong *selection_time, gboolean *total_more, gboolean *selection_more);
 void playlist_select_all(gboolean set);
 void playlist_select_range(int min, int max, gboolean sel);
@@ -87,6 +91,7 @@ void playlist_select_invert_all(void);
 gboolean playlist_select_invert(int num);
 gboolean playlist_read_info_selection(void);
 void playlist_read_info(int pos);
+void playlist_set_shuffle(gboolean shuffle);
 
 
 #define PL_LOCK()    pthread_mutex_lock(&playlist_mutex)
