@@ -23,30 +23,43 @@
 
 GtkWidget *playlistwin;
 static GtkWidget *playlistwin_url_window = NULL;
-GtkItemFactory *playlistwin_sort_menu, *playlistwin_sub_menu, *playlistwin_popup_menu;
+static GtkItemFactory *playlistwin_sort_menu, *playlistwin_sub_menu;
+static GtkItemFactory *playlistwin_popup_menu, *playlistwin_save_menu;
 
-GdkPixmap *playlistwin_bg;
-GdkBitmap *playlistwin_mask = NULL;
-GdkGC *playlistwin_gc;
-gboolean playlistwin_focus = FALSE, playlistwin_resizing = FALSE;
-gint playlistwin_resize_x, playlistwin_resize_y, playlistwin_move_x, playlistwin_move_y;
+static GdkPixmap *playlistwin_bg;
+static GdkBitmap *playlistwin_mask;
+static GdkGC *playlistwin_gc;
+static int playlistwin_resizing, playlistwin_resize_x, playlistwin_resize_y;
+static int playlistwin_save_type;
+
+gboolean playlistwin_focus = FALSE;
 
 PlayList_List *playlistwin_list = NULL;
-PlaylistSlider *playlistwin_slider = NULL;
 PButton *playlistwin_shade, *playlistwin_close;
-TextBox *playlistwin_time_min, *playlistwin_time_sec, *playlistwin_info = NULL,
-       *playlistwin_sinfo = NULL;
-SButton *playlistwin_srew, *playlistwin_splay, *playlistwin_spause, *playlistwin_sstop;
-SButton *playlistwin_sfwd, *playlistwin_seject;
-SButton *playlistwin_sscroll_up, *playlistwin_sscroll_down;
-extern TButton *mainwin_pl;
-
-GList *playlistwin_wlist = NULL;
-
+static PlaylistSlider *playlistwin_slider = NULL;
+static TextBox *playlistwin_time_min, *playlistwin_time_sec;
+static TextBox *playlistwin_info, *playlistwin_sinfo;
+static SButton *playlistwin_srew, *playlistwin_splay;
+static SButton *playlistwin_spause, *playlistwin_sstop;
+static SButton *playlistwin_sfwd, *playlistwin_seject;
+static SButton *playlistwin_sscroll_up, *playlistwin_sscroll_down;
 Vis *playlistwin_vis;
 
+static GList *playlistwin_wlist = NULL;
 static gboolean playlistwin_vis_enabled = FALSE;
 
+static struct {
+	int w, h;
+} playlistwin_resizeq = {-1, -1};
+
+struct overwrite_data {
+	char *filename;
+	int pls;
+	GtkWidget *fsel;
+};
+
+extern TButton *mainwin_pl;
+
 enum
 {
 	ADD_URL, ADD_DIR, ADD_FILE,
@@ -59,6 +72,7 @@ enum
 
 void playlistwin_sort_menu_callback(gpointer cb_data, guint action, GtkWidget * w);
 void playlistwin_sub_menu_callback(gpointer cb_data, guint action, GtkWidget * w);
+void playlistwin_save_type_cb(gpointer cb_data, guint action, GtkWidget * w);
 void playlistwin_set_hints(void);
 
 enum
@@ -73,24 +87,49 @@ enum
 GtkItemFactoryEntry playlistwin_sort_menu_entries[] =
 {
 	{N_("/Sort List"), NULL, NULL, 0, "<Branch>"},
-	{N_("/Sort List/By Title"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_BYTITLE, "<Item>"},
-	{N_("/Sort List/By Filename"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_BYFILENAME, "<Item>"},
-	{N_("/Sort List/By Path + Filename"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_BYPATH, "<Item>"},
-	{N_("/Sort List/By Date"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_BYDATE, "<Item>"},
+	{N_("/Sort List/By Title"), NULL, playlistwin_sort_menu_callback,
+					PLAYLISTWIN_SORT_BYTITLE, "<Item>"},
+	{N_("/Sort List/By Filename"), NULL, playlistwin_sort_menu_callback,
+	 PLAYLISTWIN_SORT_BYFILENAME, "<Item>"},
+	{N_("/Sort List/By Path + Filename"), NULL, playlistwin_sort_menu_callback,
+					PLAYLISTWIN_SORT_BYPATH, "<Item>"},
+	{N_("/Sort List/By Date"), NULL, playlistwin_sort_menu_callback,
+					PLAYLISTWIN_SORT_BYDATE, "<Item>"},
 	{N_("/Sort Selection"), NULL, NULL, 0, "<Branch>"},
-	{N_("/Sort Selection/By Title"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_SEL_BYTITLE, "<Item>"},
-	{N_("/Sort Selection/By Filename"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_SEL_BYFILENAME, "<Item>"},
-	{N_("/Sort Selection/By Path + Filename"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_SEL_BYPATH, "<Item>"},
-	{N_("/Sort Selection/By Date"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_SEL_BYDATE, "<Item>"},
-	{N_("/-"), NULL, NULL, 0, "<Separator>"},
-	{N_("/Randomize List"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_RANDOMIZE, "<Item>"},
-	{N_("/Reverse List"), NULL, playlistwin_sort_menu_callback, PLAYLISTWIN_SORT_REVERSE, "<Item>"},
+	{N_("/Sort Selection/By Title"), NULL, playlistwin_sort_menu_callback,
+					PLAYLISTWIN_SORT_SEL_BYTITLE, "<Item>"},
+	{N_("/Sort Selection/By Filename"), NULL, playlistwin_sort_menu_callback,
+					PLAYLISTWIN_SORT_SEL_BYFILENAME, "<Item>"},
+	{N_("/Sort Selection/By Path + Filename"), NULL, playlistwin_sort_menu_callback,
+					PLAYLISTWIN_SORT_SEL_BYPATH, "<Item>"},
+	{N_("/Sort Selection/By Date"), NULL, playlistwin_sort_menu_callback,
+					PLAYLISTWIN_SORT_SEL_BYDATE, "<Item>"},
+	{"/-", NULL, NULL, 0, "<Separator>"},
+	{N_("/Randomize List"), NULL, playlistwin_sort_menu_callback,
+					PLAYLISTWIN_SORT_RANDOMIZE, "<Item>"},
+	{N_("/Reverse List"), NULL, playlistwin_sort_menu_callback,
+					PLAYLISTWIN_SORT_REVERSE, "<Item>"},
 };
 
 static const int playlistwin_sort_menu_entries_num = 
 	sizeof(playlistwin_sort_menu_entries) / 
 	sizeof(playlistwin_sort_menu_entries[0]);
 
+enum {
+	PLAYLISTWIN_SAVE_EXTENSION = 0, PLAYLISTWIN_SAVE_M3U, PLAYLISTWIN_SAVE_PLS
+};
+
+static GtkItemFactoryEntry playlistwin_playlist_filetypes[] = {
+	{N_("/By extension"), NULL, playlistwin_save_type_cb, PLAYLISTWIN_SAVE_EXTENSION, NULL},
+	{"/-", NULL, NULL, 0, "<Separator>"},
+	{"/m3u", NULL, playlistwin_save_type_cb, PLAYLISTWIN_SAVE_M3U, NULL},
+	{"/pls", NULL, playlistwin_save_type_cb, PLAYLISTWIN_SAVE_PLS, NULL},
+};
+
+static const int playlistwin_playlist_filetypes_num =
+	sizeof(playlistwin_playlist_filetypes) /
+	sizeof(playlistwin_playlist_filetypes[0]);
+
 enum
 {
 	PLAYLISTWIN_REMOVE_DEAD_FILES, PLAYLISTWIN_PHYSICALLY_DELETE
@@ -188,7 +227,6 @@ void playlistwin_update_sinfo(void)
 
 	if (!title)
 	{
-		PL_UNLOCK();
 		textbox_set_text(playlistwin_sinfo, "");
 		return;
 	}
@@ -260,7 +298,7 @@ gint playlistwin_get_toprow(void)
 	return (-1);
 }
 
-void playlistwin_set_toprow(gint toprow)
+void playlistwin_set_toprow(int toprow)
 {
 	if (playlistwin_list)
 		playlistwin_list->pl_first = toprow;
@@ -269,14 +307,12 @@ void playlistwin_set_toprow(gint toprow)
 
 void playlistwin_update_list(void)
 {
-	if (playlistwin_list)
-		draw_widget(playlistwin_list);
-	if (playlistwin_slider)
-		draw_widget(playlistwin_slider);
-	if (playlistwin_info)
-		playlistwin_update_info();
-	if (playlistwin_sinfo)
-		playlistwin_update_sinfo();
+	g_return_if_fail(playlistwin_list != NULL);
+
+	draw_widget(playlistwin_list);
+	draw_widget(playlistwin_slider);
+	playlistwin_update_info();
+	playlistwin_update_sinfo();
 }
 
 void playlistwin_create_mask(void)
@@ -491,6 +527,28 @@ static void playlistwin_resize(int width, int height)
 	gdk_pixmap_unref(oldbg);
 }
 
+static gboolean playlistwin_resize_handler(gpointer data)
+{
+	GDK_THREADS_ENTER();
+	playlistwin_resize(playlistwin_resizeq.w, playlistwin_resizeq.h);
+	
+	playlistwin_resizeq.w = -1;
+	playlistwin_resizeq.h = -1;
+
+	GDK_THREADS_LEAVE();
+	return FALSE;
+}
+
+static void playlistwin_queue_resize(int width, int height)
+{
+	if (playlistwin_resizeq.w == -1)
+		g_idle_add_full(G_PRIORITY_HIGH_IDLE,
+				playlistwin_resize_handler, NULL, NULL);
+
+	playlistwin_resizeq.w = width;
+	playlistwin_resizeq.h = height;
+}
+
 void playlistwin_motion(GtkWidget * widget, GdkEventMotion * event, gpointer callback_data)
 {
 	XEvent ev;
@@ -596,7 +654,7 @@ static void playlistwin_fileinfo(void)
 	 * Show the first selected file, or the
 	 * current file if nothing is selected
 	 */
-	GList *list = playlist_get_selected_list();
+	GList *list = playlist_get_selected();
 	if (list != NULL)
 	{
 		playlist_fileinfo(GPOINTER_TO_INT(list->data));
@@ -608,12 +666,10 @@ static void playlistwin_fileinfo(void)
 
 static void playlistwin_set_sensitive_sortmenu(void)
 {
-	GList *list = playlist_get_selected_list();
 	GtkWidget *w;
 	gboolean set;
 
-	set = g_list_length(list) > 1;
-	g_list_free(list);
+	set = playlist_get_num_selected() > 1;
 
 	w = gtk_item_factory_get_widget(playlistwin_sort_menu,
 					"/Sort Selection/By Title");
@@ -629,25 +685,159 @@ static void playlistwin_set_sensitive_sortmenu(void)
 	gtk_widget_set_sensitive(w, set);
 }
 
-void playlistwin_save_filesel_ok(GtkWidget * w, GtkWidget * filesel)
+void playlistwin_save_playlist_error(char* path, GtkWidget *filesel)
 {
-	gchar *filename, *text, *tmp;
+	GtkWidget *dialog, *label, *bbox, *close;
+	char *text;
 
-	if (util_filebrowser_is_dir(GTK_FILE_SELECTION(filesel)))
+	dialog = gtk_dialog_new();
+	gtk_window_set_title(GTK_WINDOW(dialog), _("Unable to write playlist!"));
+	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(filesel));
+
+	text = g_strdup_printf(_("Error writing playlist \"%s\": %s"),
+			       path, strerror(errno));
+	label = gtk_label_new(text);
+	g_free(text);
+	gtk_misc_set_padding(GTK_MISC(label), 10, 10);
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
+
+	bbox = gtk_hbutton_box_new();
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_SPREAD);
+	gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), bbox, FALSE, FALSE, 0);
+
+	close = gtk_button_new_with_label(_("Ok"));
+	gtk_signal_connect_object(GTK_OBJECT(close), "clicked",
+				  GTK_SIGNAL_FUNC(gtk_widget_destroy),
+				  GTK_OBJECT(dialog));
+	GTK_WIDGET_SET_FLAGS(close, GTK_CAN_DEFAULT);
+	gtk_box_pack_start(GTK_BOX(bbox), close, FALSE, FALSE, 0);
+	gtk_widget_grab_default(close);
+
+	gtk_widget_show_all(dialog);
+}	
+
+static void playlistwin_check_overwrite_cb(GtkButton *w, gpointer user_data)
+{
+	struct overwrite_data *odata = user_data;
+	if (!playlist_save(odata->filename, odata->pls))
+		playlistwin_save_playlist_error(odata->filename, odata->fsel);
+	else
+	{
+		gtk_widget_hide(odata->fsel);
+		g_free(user_data);
+	}
+}
+
+static void playlistwin_check_overwrite(GtkWidget *filesel, char *filename, int pls)
+{
+	GtkWidget *dialog, *label, *bbox, *overwrite, *cancel;
+	char *text;
+	struct overwrite_data *data;
+
+	data = g_malloc0(sizeof (*data));
+	data->filename = filename;
+	data->pls = pls;
+	data->fsel = filesel;
+	
+	dialog = gtk_dialog_new();
+	gtk_window_set_title(GTK_WINDOW(dialog), _("File exists!"));
+	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(filesel));
+
+	text = g_strdup_printf(_("%s already exist."), filename);
+	label = gtk_label_new(text);
+	gtk_misc_set_padding(GTK_MISC(label), 10, 10);
+	g_free(text);
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
+
+	bbox = gtk_hbutton_box_new();
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_SPREAD);
+	gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), bbox, FALSE, FALSE, 0);
+
+	overwrite = gtk_button_new_with_label(_("Overwrite"));
+	gtk_signal_connect(GTK_OBJECT(overwrite), "clicked",
+			   GTK_SIGNAL_FUNC(playlistwin_check_overwrite_cb),
+			   data);	
+	gtk_signal_connect_object(GTK_OBJECT(overwrite), "clicked",
+				  GTK_SIGNAL_FUNC(gtk_widget_destroy),
+				  GTK_OBJECT(dialog));	
+	GTK_WIDGET_SET_FLAGS(overwrite, GTK_CAN_DEFAULT);
+	gtk_box_pack_start(GTK_BOX(bbox), overwrite, FALSE, FALSE, 0);
+	cancel = gtk_button_new_with_label(_("Cancel"));
+	gtk_signal_connect_object(GTK_OBJECT(cancel), "clicked",
+				  GTK_SIGNAL_FUNC(g_free),
+				  (GtkObject*) data);
+	gtk_signal_connect_object(GTK_OBJECT(cancel), "clicked",
+				  GTK_SIGNAL_FUNC(gtk_widget_destroy),
+				  GTK_OBJECT(dialog));
+	GTK_WIDGET_SET_FLAGS(cancel, GTK_CAN_DEFAULT);
+	gtk_box_pack_start(GTK_BOX(bbox), cancel, FALSE, FALSE, 0);
+	gtk_widget_grab_default(overwrite);
+
+	gtk_widget_show_all(dialog);
+}
+
+void playlistwin_save_filesel_ok(GtkWidget * w, GtkFileSelection * filesel)
+{
+	char *filename, *slash;
+	struct stat statd;
+	int len;
+	gboolean pls = FALSE;
+
+	if (util_filebrowser_is_dir(filesel))
 		return;
 	
-	filename = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel)));
-	text = g_strdup(filename);
+	filename = gtk_file_selection_get_filename(filesel);
+	
+	if ((slash = strrchr(filename, '/')) != NULL)
+		len = slash - filename + 1;
+	else
+		len = strlen(filename);
 
-	if ((tmp = strrchr(text, '/')) != NULL)
-		*(tmp + 1) = '\0';
+	if (playlistwin_save_type == PLAYLISTWIN_SAVE_EXTENSION)
+	{
+		char *ext;
+		int error = FALSE;
+		if ((ext = strrchr(filename, '.')) != NULL && ext > slash)
+		{
+			if (!strcmp(ext, ".pls"))
+				pls = TRUE;
+			else if (strcmp(ext, ".m3u"))
+				error = TRUE;
+		}
+		else
+			error = TRUE;
+		if (error)
+		{
+			char *m;
+			GtkWidget *w;
+			m = g_strdup_printf(_("Unknown filetype for %s"),
+					    filename);
+			w = xmms_show_message(_("Unable to save playlist"), m,
+					      _("Ok"), TRUE, NULL, NULL);
+			gtk_window_set_transient_for(GTK_WINDOW(filesel),
+						     GTK_WINDOW(w));
+
+			g_free(m);
+			return;
+		}
+	}
+	else if (playlistwin_save_type == PLAYLISTWIN_SAVE_PLS)
+		pls = TRUE;
+	
 	g_free(cfg.playlist_path);
-	cfg.playlist_path = g_strdup(text);
-	g_free(text);
+	cfg.playlist_path = g_strndup(filename, len);
 
-	if (filename && *filename)
-		playlist_save(filename);
-	gtk_widget_destroy(GTK_WIDGET(filesel));
+	if (stat(filename, &statd) == 0)
+		playlistwin_check_overwrite(GTK_WIDGET(filesel), filename, pls);
+	else
+	{
+		if (!playlist_save(filename, pls))
+			playlistwin_save_playlist_error(filename, GTK_WIDGET(filesel));
+		else
+			gtk_widget_hide(GTK_WIDGET(filesel));
+	}
 }
 
 void playlistwin_load_filesel_ok(GtkWidget * w, GtkWidget * filesel)
@@ -680,14 +870,11 @@ void playlistwin_load_filesel_ok(GtkWidget * w, GtkWidget * filesel)
 static void playlistwin_show_sub_misc_menu(void)
 {
 	gint x, y;
-	GList *list;
 	GtkWidget *widget;
 
-	list = playlist_get_selected_list();
 	widget = gtk_item_factory_get_widget(playlistwin_sub_menu,
 					     "/Physically Delete Files");
-	gtk_widget_set_sensitive(widget, g_list_length(list) > 0);
-	g_list_free(list);
+	gtk_widget_set_sensitive(widget, playlist_get_num_selected() > 0);
 	gdk_window_get_pointer(NULL, &x, &y, NULL);
 	util_item_factory_popup(GTK_ITEM_FACTORY(playlistwin_sub_menu),
 				x, y, 1, GDK_CURRENT_TIME);
@@ -720,28 +907,56 @@ static void playlistwin_show_load_filesel(void)
 
 static void playlistwin_show_save_filesel(void)
 {
-	static GtkWidget *save_filesel;
+	static GtkWidget *filesel;
+	GtkWidget *frame, *hbox, *label, *menu, *om;
 	GtkObject *object;
 
-	if (save_filesel != NULL)
+	if (filesel != NULL)
+	{
+		if (!GTK_WIDGET_VISIBLE(filesel))
+		{
+			if (cfg.playlist_path)
+				gtk_file_selection_set_filename(
+					GTK_FILE_SELECTION(filesel),
+					cfg.playlist_path);
+			gtk_widget_show(filesel);
+		}
 		return;
+	}
 
-	save_filesel = gtk_file_selection_new(_("Save playlist"));
+	filesel = gtk_file_selection_new(_("Save playlist"));
 	if (cfg.playlist_path)
-		gtk_file_selection_set_filename(GTK_FILE_SELECTION(save_filesel),
+		gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel),
 						cfg.playlist_path);
-	object = GTK_OBJECT(GTK_FILE_SELECTION(save_filesel)->ok_button);
+	object = GTK_OBJECT(GTK_FILE_SELECTION(filesel)->ok_button);
 	gtk_signal_connect(object, "clicked",
 			   GTK_SIGNAL_FUNC(playlistwin_save_filesel_ok),
-			   save_filesel);
-	object = GTK_OBJECT(GTK_FILE_SELECTION(save_filesel)->cancel_button);
+			   filesel);
+	object = GTK_OBJECT(GTK_FILE_SELECTION(filesel)->cancel_button);
 	gtk_signal_connect_object(object, "clicked",
-				  GTK_SIGNAL_FUNC(gtk_widget_destroy),
-				  GTK_OBJECT(save_filesel));
-	gtk_signal_connect(GTK_OBJECT(save_filesel), "destroy",
-			   GTK_SIGNAL_FUNC(gtk_widget_destroyed),
-			   &save_filesel);
-	gtk_widget_show(save_filesel);
+				  GTK_SIGNAL_FUNC(gtk_widget_hide),
+				  GTK_OBJECT(filesel));
+	gtk_signal_connect(GTK_OBJECT(filesel), "destroy",
+			   GTK_SIGNAL_FUNC(gtk_widget_destroyed), &filesel);
+	frame = gtk_frame_new(_("Save options"));
+	hbox = gtk_hbox_new(FALSE, 5);
+	gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
+	gtk_container_add(GTK_CONTAINER(frame), hbox);
+	label = gtk_label_new(_("Determine file type:"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+	om = gtk_option_menu_new();
+	gtk_box_pack_start(GTK_BOX(hbox), om, TRUE, TRUE, 0);
+	menu = gtk_item_factory_get_widget(playlistwin_save_menu, "<Save>");
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(om), menu);
+	gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(filesel)->action_area),
+			   frame, TRUE, TRUE, 0);
+
+	gtk_widget_show_all(filesel);
+}
+
+void playlistwin_save_type_cb(gpointer cb_data, guint action, GtkWidget * w)
+{
+	playlistwin_save_type = action;
 }
 
 void playlistwin_popup_handler(gint item)
@@ -849,38 +1064,8 @@ static gboolean inside_sensitive_widgets(gint x, gint y)
 
 void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpointer callback_data)
 {
-	gint add_nx[] =	{0, 0, 0},
-	add_ny[] = {111, 130, 149},
-	add_sx[] = {23, 23, 23},
-	add_sy[] = {111, 130, 149},
-	add_barx = 48, add_bary = 111;
-	
-	gint sub_nx[] =	{54, 54, 54, 54},
-	sub_ny[] = {168, 111, 130, 149},
-	sub_sx[] = {77, 77, 77, 77},
-	sub_sy[] = {168, 111, 130, 149},
-	sub_barx = 100, sub_bary = 111;
-	
-	gint sel_nx[] =	{104, 104, 104},
-	sel_ny[] = {111, 130, 149},
-	sel_sx[] = {127, 127, 127},
-	sel_sy[] = {111, 130, 149},
-	sel_barx = 150, sel_bary = 111;
-	
-	gint misc_nx[] = {154, 154, 154},
-	misc_ny[] = {111, 130, 149},
-	misc_sx[] = {177, 177, 177},
-	misc_sy[] = {111, 130, 149},
-	misc_barx = 200, misc_bary = 111;
-	
-	gint plist_nx[] = {204, 204, 204},
-	plist_ny[] = {111, 130, 149},
-	plist_sx[] = {227, 227, 227},
-	plist_sy[] = {111, 130, 149},
-	plist_barx = 250, plist_bary = 111;
-
 	gboolean grab = TRUE;
-	gint xpos, ypos;
+	int xpos, ypos;
 
 	dock_get_widget_pos(playlistwin, &xpos, &ypos);
 	if (event->button == 1 && !cfg.show_wm_decorations &&
@@ -890,49 +1075,78 @@ void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpointer call
 	      event->x >= cfg.playlist_width - 31 &&
 	      event->x < cfg.playlist_width - 22)))
 	{
-		playlistwin_resizing = TRUE;
-		playlistwin_resize_x = cfg.playlist_width - event->x;
-		playlistwin_resize_y = cfg.playlist_height - event->y;
-		playlistwin_raise();
+		if (hint_move_resize_available())
+		{
+			hint_move_resize(playlistwin, event->x_root,
+					 event->y_root, FALSE);
+			grab = FALSE;
+		}
+		else
+		{
+			playlistwin_resizing = TRUE;
+			playlistwin_resize_x = cfg.playlist_width - event->x;
+			playlistwin_resize_y = cfg.playlist_height - event->y;
+			playlistwin_raise();
+		}
 	}
 	else if (event->button == 1 && REGION_L(12, 37, 29, 11))
 	{
+		int nx[] = {0, 0, 0}, ny[] = {111, 130, 149};
+		int sx[] = {23, 23, 23}, sy[] = {111, 130, 149};
+		int barx = 48, bary = 111;
+
 		playlist_popup(xpos + 12,
-			       ypos + cfg.playlist_height - (3 * 18) - 11,
-			       3, add_nx, add_ny, add_sx, add_sy, add_barx, add_bary,
+			       ypos + cfg.playlist_height - (3 * 18) - 11, 3,
+			       nx, ny, sx, sy, barx, bary,
 			       ADD_URL, playlistwin_popup_handler);
 		grab = FALSE;
 	}
 	else if (event->button == 1 && REGION_L(41, 66, 29, 11))
 	{
+		int nx[] = {54, 54, 54, 54}, ny[] = {168, 111, 130, 149};
+		int sx[] = {77, 77, 77, 77}, sy[] = {168, 111, 130, 149};
+		int barx = 100, bary = 111;
+
 		playlist_popup(xpos + 41,
-			       ypos + cfg.playlist_height - (4 * 18) - 11,
-			       4, sub_nx, sub_ny, sub_sx, sub_sy, sub_barx, sub_bary,
+			       ypos + cfg.playlist_height - (4 * 18) - 11,  4,
+			       nx, ny, sx, sy, barx, bary,
 			       SUB_MISC, playlistwin_popup_handler);
 		grab = FALSE;
 	}
 	else if (event->button == 1 && REGION_L(70, 95, 29, 11))
 	{
+		int nx[] = {104, 104, 104}, ny[] = {111, 130, 149};
+		int sx[] = {127, 127, 127}, sy[] = {111, 130, 149};
+		int barx = 150, bary = 111;
+	
 		playlist_popup(xpos + 70,
-			       ypos + cfg.playlist_height - (3 * 18) - 11,
-			       3, sel_nx, sel_ny, sel_sx, sel_sy, sel_barx, sel_bary,
+			       ypos + cfg.playlist_height - (3 * 18) - 11, 3,
+			       nx, ny, sx, sy, barx, bary,
 			       SEL_INV, playlistwin_popup_handler);
 		grab = FALSE;
 	}
 	else if (event->button == 1 && REGION_L(99, 124, 29, 11))
 	{
+		int nx[] = {154, 154, 154}, ny[] = {111, 130, 149};
+		int sx[] = {177, 177, 177}, sy[] = {111, 130, 149};
+		int barx = 200, bary = 111;
+	
 		playlist_popup(xpos + 99,
-			       ypos + cfg.playlist_height - (3 * 18) - 11,
-			       3, misc_nx, misc_ny, misc_sx, misc_sy, misc_barx,
-			       misc_bary, MISC_SORT, playlistwin_popup_handler);
+			       ypos + cfg.playlist_height - (3 * 18) - 11, 3,
+			       nx, ny, sx, sy, barx, bary,
+			       MISC_SORT, playlistwin_popup_handler);
 		grab = FALSE;
 	}
 	else if (event->button == 1 && REGION_R(46, 23, 29, 11))
 	{
+		int nx[] = {204, 204, 204}, ny[] = {111, 130, 149};
+		int sx[] = {227, 227, 227}, sy[] = {111, 130, 149};
+		int barx = 250, bary = 111;
+
 		playlist_popup(xpos + cfg.playlist_width - 46,
-			       ypos + cfg.playlist_height - (3 * 18) - 11,
-			       3, plist_nx, plist_ny, plist_sx, plist_sy, plist_barx,
-			       plist_bary, PLIST_NEW, playlistwin_popup_handler);
+			       ypos + cfg.playlist_height - (3 * 18) - 11, 3,
+			       nx, ny, sx, sy, barx, bary,
+			       PLIST_NEW, playlistwin_popup_handler);
 		grab = FALSE;
 	}
 	else if (event->button == 1 && REGION_R(82, 54, 15, 9))
@@ -971,9 +1185,18 @@ void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpointer call
 		 !inside_sensitive_widgets(event->x, event->y) &&
 		 (cfg.easy_move || event->y < 14))
 	{
-		gdk_window_raise(playlistwin->window);
-		dock_move_press(dock_window_list, playlistwin, event, FALSE);
-
+		if (0 && hint_move_resize_available())
+		{
+			hint_move_resize(playlistwin, event->x_root,
+					 event->y_root, TRUE);
+			grab = FALSE;
+		}
+		else
+		{
+			gdk_window_raise(playlistwin->window);
+			dock_move_press(dock_window_list, playlistwin,
+					event, FALSE);
+		}
 	}
 	else if (event->button == 1 && event->type == GDK_2BUTTON_PRESS &&
 		 !inside_sensitive_widgets(event->x, event->y) && event->y < 14)
@@ -1056,15 +1279,17 @@ static gboolean playlistwin_configure(GtkWidget * window, GdkEventConfigure *eve
 {
 	if (!GTK_WIDGET_VISIBLE(window))
 		return FALSE;
-	
-	if (cfg.show_wm_decorations)
+
+	if (cfg.show_wm_decorations || hint_move_resize_available())
 	{
 		if (event->width != cfg.playlist_width ||
 		    event->height != cfg.playlist_height)
-			playlistwin_resize(event->width, event->height);
+			playlistwin_queue_resize(event->width, event->height);
+	}
+	
+	if (cfg.show_wm_decorations)
 		gdk_window_get_root_origin(window->window,
 					   &cfg.playlist_x, &cfg.playlist_y);
-	}
 	else
 		gdk_window_get_deskrelative_origin(window->window,
 						   &cfg.playlist_x,
@@ -1107,7 +1332,7 @@ void playlistwin_set_hints(void)
 	GdkGeometry geometry;
 	GdkWindowHints mask;
 
-	if (!cfg.show_wm_decorations)
+	if (!cfg.show_wm_decorations && !hint_move_resize_available())
 		return;
 
 	geometry.min_width = 275;
@@ -1162,7 +1387,7 @@ void playlistwin_physically_delete_cb(GtkWidget *widget, gpointer data)
 	}
 
 	playlist_delete_filenames(selected_list);
-	g_list_foreach(selected_list, (GFunc)g_free_func, NULL);
+	g_list_foreach(selected_list, (GFunc)g_free, NULL);
 	g_list_free(selected_list);
 
 	if (length > 1 && deleted < length)
@@ -1183,7 +1408,7 @@ void playlistwin_physically_delete(void)
 	gint length;
 
 	PL_LOCK();
-	for (node = get_playlist(); node != 0; node = g_list_next(node))
+	for (node = get_playlist(); node != NULL; node = g_list_next(node))
 	{
 		PlaylistEntry *entry = node->data;
 		/*
@@ -1237,6 +1462,66 @@ void playlistwin_physically_delete(void)
 	gtk_widget_show_all(dialog);
 }
 
+static void playlistwin_keypress_up_down_handler(PlayList_List *pl, gboolean up, guint state)
+{
+	if ((state & GDK_MOD1_MASK) && (state & GDK_SHIFT_MASK))
+		return;
+	if (!(state & GDK_MOD1_MASK))
+		playlist_select_all(FALSE);
+	
+	if (pl->pl_prev_selected == -1 ||
+	    (!playlistwin_item_visible(pl->pl_prev_selected) &&
+	     !(state & GDK_SHIFT_MASK && pl->pl_prev_min != -1)))
+	{
+		pl->pl_prev_selected = pl->pl_first;
+	}
+	else if (state & GDK_SHIFT_MASK)
+	{
+		if (pl->pl_prev_min == -1)
+		{
+			pl->pl_prev_max = pl->pl_prev_selected;
+			pl->pl_prev_min = pl->pl_prev_selected;
+		}
+		pl->pl_prev_max += (up ? -1 : 1);
+		pl->pl_prev_max =
+			CLAMP(pl->pl_prev_max, 0, get_playlist_length() - 1);
+		
+		pl->pl_first = MIN(pl->pl_first, pl->pl_prev_max);
+		pl->pl_first = MAX(pl->pl_first, pl->pl_prev_max -
+				   pl->pl_num_visible + 1);
+		playlist_select_range(pl->pl_prev_min, pl->pl_prev_max, TRUE);
+		return;
+	}
+	else if (state & GDK_MOD1_MASK)
+	{
+		if (up)
+			playlist_list_move_up(pl);
+		else
+			playlist_list_move_down(pl);
+		if (pl->pl_prev_min < pl->pl_first)
+			pl->pl_first = pl->pl_prev_min;
+		else if (pl->pl_prev_max >= (pl->pl_first + pl->pl_num_visible))
+			pl->pl_first = pl->pl_prev_max - pl->pl_num_visible + 1;
+		return;
+	}
+	else if (up)
+		pl->pl_prev_selected--;
+	else
+		pl->pl_prev_selected++;
+	
+	pl->pl_prev_selected =
+		CLAMP(pl->pl_prev_selected, 0, get_playlist_length() - 1);
+	
+	if (pl->pl_prev_selected < pl->pl_first)
+		pl->pl_first--;
+	else if (pl->pl_prev_selected >= (pl->pl_first + pl->pl_num_visible))
+		pl->pl_first++;
+	
+	playlist_select_range(pl->pl_prev_selected, pl->pl_prev_selected, TRUE);
+	pl->pl_prev_min = -1;
+
+}
+
 gboolean playlistwin_keypress(GtkWidget * w, GdkEventKey * event, gpointer data)
 {
 	guint keyval;
@@ -1252,61 +1537,12 @@ gboolean playlistwin_keypress(GtkWidget * w, GdkEventKey * event, gpointer data)
 	{
 		case GDK_KP_Up:
 		case GDK_KP_Down:
-			keyval = event->keyval == GDK_KP_Up ? GDK_Up : GDK_Down;
 		case GDK_Up:
 		case GDK_Down:
-			if ((event->state & GDK_MOD1_MASK) && (event->state & GDK_SHIFT_MASK))
-				break;
-			if (event->state & GDK_MOD1_MASK)
-			{
-				if (playlistwin_list->pl_prev_selected > -1)
-				{
-					if (keyval == GDK_Up)
-						playlist_list_move_up(playlistwin_list);
-					else
-						playlist_list_move_down(playlistwin_list);
-				}
-			}
-			else
-				playlist_select_all(FALSE);
-
-			if (playlistwin_list->pl_prev_selected == -1 ||
-			    (!playlistwin_item_visible(playlistwin_list->pl_prev_selected) &&
-			     !(event->state & GDK_SHIFT_MASK && playlistwin_list->pl_prev_min != -1)))
-			{
-				playlistwin_list->pl_prev_selected = playlistwin_list->pl_first;
-			}
-			else if (event->state & GDK_SHIFT_MASK)
-			{
-				if (playlistwin_list->pl_prev_min == -1)
-				{
-					playlistwin_list->pl_prev_max = playlistwin_list->pl_prev_selected;
-					playlistwin_list->pl_prev_min = playlistwin_list->pl_prev_selected;
-				}
-				playlistwin_list->pl_prev_max += (keyval == GDK_Up ? -1 : 1);
-				playlistwin_list->pl_prev_max = CLAMP(playlistwin_list->pl_prev_max, 0, get_playlist_length() - 1);
-
-				playlistwin_list->pl_first = MIN(playlistwin_list->pl_first, playlistwin_list->pl_prev_max);
-				playlistwin_list->pl_first = MAX(playlistwin_list->pl_first, playlistwin_list->pl_prev_max - playlistwin_list->pl_num_visible + 1);
-				playlist_select_range(playlistwin_list->pl_prev_min, playlistwin_list->pl_prev_max, TRUE);
-				break;
-			}
-			else if (keyval == GDK_Up)
-				playlistwin_list->pl_prev_selected--;
-			else
-				playlistwin_list->pl_prev_selected++;
-
-			playlistwin_list->pl_prev_selected = CLAMP(playlistwin_list->pl_prev_selected, 0, get_playlist_length() - 1);
-
-			if (playlistwin_list->pl_prev_selected < playlistwin_list->pl_first)
-				playlistwin_list->pl_first--;
-			else if (playlistwin_list->pl_prev_selected >= (playlistwin_list->pl_first + playlistwin_list->pl_num_visible))
-				playlistwin_list->pl_first++;
-
-			if (event->state & GDK_MOD1_MASK)
-				break;
-			playlist_select_range(playlistwin_list->pl_prev_selected, playlistwin_list->pl_prev_selected, TRUE);
-			playlistwin_list->pl_prev_min = -1;
+			playlistwin_keypress_up_down_handler(
+				playlistwin_list,
+				keyval == GDK_Up || keyval == GDK_KP_Up,
+				event->state);
 			break;
 		case GDK_Page_Up:
 			playlistwin_scroll(-playlistwin_list->pl_num_visible);
@@ -1335,7 +1571,7 @@ gboolean playlistwin_keypress(GtkWidget * w, GdkEventKey * event, gpointer data)
 			refresh = FALSE;
 			break;
 		case GDK_Delete:
-			if (event->state & GDK_SHIFT_MASK)
+			if (event->state & GDK_CONTROL_MASK)
 				playlist_delete(TRUE);
 			else
 				playlist_delete(FALSE);
@@ -1725,6 +1961,14 @@ void playlistwin_create(void)
 	menu = gtk_item_factory_get_widget(playlistwin_sub_menu, "");
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
 
+	playlistwin_save_menu =
+		gtk_item_factory_new(GTK_TYPE_MENU, "<Save>", NULL);
+	gtk_item_factory_set_translate_func(playlistwin_save_menu,
+					    util_menu_translate, NULL, NULL);
+	gtk_item_factory_create_items(GTK_ITEM_FACTORY(playlistwin_save_menu),
+				      playlistwin_playlist_filetypes_num,
+				      playlistwin_playlist_filetypes, NULL);
+
 	playlistwin_create_gtk();
 	playlistwin_gc = gdk_gc_new(playlistwin->window);
 	playlistwin_create_widgets();
