@@ -19,7 +19,7 @@
 #ifndef SM_H
 #define SM_H
 
-void sm_init(int argc, char **argv);
+void sm_init(int argc, char **argv, const char *previous_session_id);
 void sm_cleanup(void);
 
 #endif
