@@ -1,6 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-2001  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999-2001  Hvard Kvlen <havardk@xmms.org>
+ *  Copyright (C) 1998-2001  Peter Alm, Mikael Alm, Olle Hallnas,
+ *                           Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1999-2001  Haavard Kvaalen <havardk@xmms.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +22,9 @@
 #include <X11/Xlib.h>
 #include <sys/ipc.h>
 #include <ctype.h>
+#ifdef HAVE_FTS_H
+#include <fts.h>
+#endif
 
 static GQuark quark_popup_data;
 
@@ -39,61 +43,106 @@ gchar *find_file_recursively(const char *dirname, const char *file)
 {
 	DIR *dir;
 	struct dirent *dirent;
-	gchar *result;
-	gchar *found;
+	char *result, *found;
 	struct stat buf;
 
-	if ((dir = opendir(dirname)) != NULL)
+	if ((dir = opendir(dirname)) == NULL)
+		return NULL;
+
+	while ((dirent = readdir(dir)) != NULL)
 	{
-		while ((dirent = readdir(dir)) != NULL)
+		/* Don't scan "." and ".." */
+		if (!strcmp(dirent->d_name, ".") ||
+		    !strcmp(dirent->d_name, ".."))
+			continue;
+		/* We need this in order to find out if file is directory */
+		found = g_strconcat(dirname, "/", dirent->d_name, NULL);
+		if (stat(found, &buf) != 0)
 		{
-			/* Don't scan "." and ".." */
-			if (strcmp(dirent->d_name, ".") == 0 || strcmp(dirent->d_name, "..") == 0)
-				continue;
-			/* We need this in order to find out if file is directory */
-			found = g_strdup_printf("%s/%s", dirname, dirent->d_name);
-			if (stat(found, &buf) != 0)
-			{	/* Error occured */
-				g_free(found);
-				closedir(dir);
-				return NULL;
-			}
-			if (S_ISDIR(buf.st_mode))
-			{	/* Found directory -- descend into it */
-				result = find_file_recursively(found, file);
-				if (result != NULL)
-				{	/* Found file there -- exit */
+			/* Error occured */
+			g_free(found);
+			closedir(dir);
+			return NULL;
+		}
+		if (!S_ISDIR(buf.st_mode))
+		{
+			/* Normal file -- maybe just what we are looking for? */
+			if (!strcasecmp(dirent->d_name, file))
+			{
+				if (strlen(found) > FILENAME_MAX)
+				{
+					/* No good! File + path too long */
 					g_free(found);
 					closedir(dir);
-					return result;
+					return NULL;
+				}
+				else
+				{
+					closedir(dir);
+					return found;
 				}
 			}
-			else
-			{	/* Normal file -- maybe just what we are looking for? */
-				if (!strcasecmp(dirent->d_name, file))
-				{	/* Yes! */
-					if (strlen(found) > FILENAME_MAX)
-					{	/* No good! File + path too long */
-						g_free(found);
-						closedir(dir);
-						return NULL;
-					}
-					else
-					{
-						closedir(dir);
-						return found;
-					}
-				}
-			}
-			g_free(found);
 		}
-		closedir(dir);
+		g_free(found);
 	}
+	rewinddir(dir);
+	while ((dirent = readdir(dir)) != NULL)
+	{
+		/* Don't scan "." and ".." */
+		if (!strcmp(dirent->d_name, ".") ||
+		    !strcmp(dirent->d_name, ".."))
+			continue;
+		/* We need this in order to find out if file is directory */
+		found = g_strconcat(dirname, "/", dirent->d_name, NULL);
+		if (stat(found, &buf) != 0)
+		{
+			/* Error occured */
+			g_free(found);
+			closedir(dir);
+			return NULL;
+		}
+		if (S_ISDIR(buf.st_mode))
+		{
+			/* Found directory -- descend into it */
+			result = find_file_recursively(found, file);
+			if (result != NULL)
+			{	/* Found file there -- exit */
+				g_free(found);
+				closedir(dir);
+				return result;
+			}
+		}
+		g_free(found);
+	}
+	closedir(dir);
 	return NULL;
 }
 
 void del_directory(const char *dirname)
 {
+#ifdef HAVE_FTS_H
+	char * const argv[2] = { (char *) dirname, NULL };
+	FTS *fts;
+	FTSENT *p;
+
+	fts = fts_open(argv, FTS_PHYSICAL, (int (*)())NULL);
+	while ((p = fts_read(fts)) != NULL) {
+		switch (p->fts_info) {
+		case FTS_D:
+			break;
+		case FTS_DNR:
+		case FTS_ERR:
+			break;
+		case FTS_DP:
+			rmdir(p->fts_accpath);
+			break;
+		default:
+			unlink(p->fts_accpath);
+			break;
+		}
+	}
+	fts_close(fts);
+#else /* !HAVE_FTS_H */
 	DIR *dir;
 	struct dirent *dirent;
 	char *file;
@@ -114,6 +163,7 @@ void del_directory(const char *dirname)
 		closedir(dir);
 	}
 	rmdir(dirname);
+#endif /* !HAVE_FTS_H */
 }
 
 GdkImage *create_dblsize_image(GdkImage * img)
@@ -232,95 +282,100 @@ GdkImage *create_dblsize_image(GdkImage * img)
 	return dblimg;
 }
 
-char *read_ini_string(const char *filename, const char *section, const char *key)
+static char *read_string(const char *filename, const char *section, const char *key, gboolean comment)
 {
 	FILE *file;
 	char *buffer, *ret_buffer = NULL;
-	int found_section = 0, found_key = 0, off = 0, len = 0;
+	int found_section = 0, off = 0, len = 0;
 	struct stat statbuf;
 
 	if (!filename)
 		return NULL;
 
-	if ((file = fopen(filename, "r")) != NULL)
+	if ((file = fopen(filename, "r")) == NULL)
+		return NULL;
+
+	if (stat(filename, &statbuf) < 0)
 	{
-		stat(filename, &statbuf);
-		buffer = (char *) g_malloc(statbuf.st_size);
-		fread(buffer, 1, statbuf.st_size, file);
-		while (!found_key && off < statbuf.st_size)
+		fclose(file);
+		return NULL;
+	}
+		
+	buffer = g_malloc(statbuf.st_size);
+	fread(buffer, 1, statbuf.st_size, file);
+	while (!ret_buffer && off < statbuf.st_size)
+	{
+		while (off < statbuf.st_size &&
+		       (buffer[off] == '\r' || buffer[off] == '\n' ||
+			buffer[off] == ' ' || buffer[off] == '\t'))
+			off++;
+		if (off >= statbuf.st_size)
+			break;
+		if (buffer[off] == '[')
 		{
-			while ((buffer[off] == '\r' || buffer[off] == '\n' ||
-				buffer[off] == ' ' || buffer[off] == '\t') &&
-			       off < statbuf.st_size)
+			int slen = strlen(section);
+			off++;
+			found_section = 0;
+			if (off + slen + 1 < statbuf.st_size &&
+			    !strncasecmp(section, &buffer[off], slen))
+			{
+				off += slen;
+				if (buffer[off] == ']')
+				{
+					off++;
+					found_section = 1;
+				}
+			}
+		}
+		else if (found_section && off + strlen(key) < statbuf.st_size &&
+			 !strncasecmp(key, &buffer[off], strlen(key)))
+		{
+			off += strlen(key);
+			while (off < statbuf.st_size &&
+			       (buffer[off] == ' ' || buffer[off] == '\t'))
 				off++;
 			if (off >= statbuf.st_size)
 				break;
-			if (buffer[off] == '[')
+			if (buffer[off] == '=')
 			{
 				off++;
+				while (off < statbuf.st_size &&
+				       (buffer[off] == ' ' ||
+					buffer[off] == '\t'))
+					off++;
 				if (off >= statbuf.st_size)
 					break;
-				if (off < statbuf.st_size - strlen(section))
-				{
-					if (!strncasecmp(section, &buffer[off], strlen(section)))
-					{
-						off += strlen(section);
-						if (off >= statbuf.st_size)
-							break;
-						if (buffer[off] == ']')
-							found_section = 1;
-						else
-							found_section = 0;
-						off++;
-						if (off >= statbuf.st_size)
-							break;
-					}
-					else
-						found_section = 0;
-				}
-				else
-					found_section = 0;
-
+				len = 0;
+				while (off + len < statbuf.st_size &&
+				       buffer[off + len] != '\r' &&
+				       buffer[off + len] != '\n' &&
+				       (!comment ||
+					buffer[off + len] != ';'))
+					len++;
+				ret_buffer = g_strndup(&buffer[off], len);
+				off += len;
 			}
-			else if (found_section)
-			{
-				if (off < statbuf.st_size - strlen(key))
-				{
-					if (!strncasecmp(key, &buffer[off], strlen(key)))
-					{
-						off += strlen(key);
-						while ((buffer[off] == ' ' || buffer[off] == '\t') && off < statbuf.st_size)
-							off++;
-						if (off >= statbuf.st_size)
-							break;
-						if (buffer[off] == '=')
-						{
-							off++;
-							while ((buffer[off] == ' ' || buffer[off] == '\t') && off < statbuf.st_size)
-								off++;
-							if (off >= statbuf.st_size)
-								break;
-							len = 0;
-							while (buffer[off + len] != '\r' && buffer[off + len] != '\n' && buffer[off + len] != ';' && off + len < statbuf.st_size)
-								len++;
-							ret_buffer = (char *) g_malloc(len + 1);
-							strncpy(ret_buffer, &buffer[off], len);
-							ret_buffer[len] = '\0';
-							off += len;
-							found_key = 1;
-						}
-					}
-				}
-			}
-			while (buffer[off] != '\r' && buffer[off] != '\n' && off < statbuf.st_size)
-				off++;
 		}
-		g_free(buffer);
-		fclose(file);
+		while (off < statbuf.st_size &&
+		       buffer[off] != '\r' && buffer[off] != '\n')
+			off++;
 	}
+
+	g_free(buffer);
+	fclose(file);
 	return ret_buffer;
 }
 
+char *read_ini_string(const char *filename, const char *section, const char *key)
+{
+	return read_string(filename, section, key, TRUE);
+}
+
+char *read_ini_string_no_comment(const char *filename, const char *section, const char *key)
+{
+	return read_string(filename, section, key, FALSE);
+}
+
 GArray *string_to_garray(const gchar * str)
 {
 	GArray *array;
@@ -443,7 +498,7 @@ void util_item_factory_popup_with_data(GtkItemFactory * ifactory,
 		pos = g_malloc0(sizeof (struct MenuPos));
 
 		gtk_object_set_data_by_id_full(GTK_OBJECT(ifactory->widget),
-					       quark_user_menu_pos, pos, g_free_func);
+					       quark_user_menu_pos, pos, g_free);
 	}
 	pos->x = x;
 	pos->y = y;
@@ -519,6 +574,8 @@ GtkWidget* util_create_add_url_window(gchar *caption, GtkSignalFunc ok_func, Gtk
 	gtk_widget_show(combo);
 	
 	bbox = gtk_hbutton_box_new();
+	gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
+	gtk_widget_show(bbox);
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
 	gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
 	
@@ -527,13 +584,13 @@ GtkWidget* util_create_add_url_window(gchar *caption, GtkSignalFunc ok_func, Gtk
 	gtk_signal_connect(GTK_OBJECT(ok), "clicked", ok_func, GTK_COMBO(combo)->entry);
 	
 	GTK_WIDGET_SET_FLAGS(ok, GTK_CAN_DEFAULT);
-	gtk_window_set_default(GTK_WINDOW(win), ok);
+	gtk_widget_grab_default(ok);
 	gtk_box_pack_start(GTK_BOX(bbox), ok, FALSE, FALSE, 0);
 	gtk_widget_show(ok);
 
-	if(enqueue_func)
+	if (enqueue_func)
 	{
-		/* I18N note: "Enqueue" here means "Add to playlist" */
+		/* I18N: "Enqueue" here means "Add to playlist" */
 		enqueue = gtk_button_new_with_label(_("Enqueue"));
 		gtk_signal_connect(GTK_OBJECT(enqueue), "clicked", util_add_url_callback, GTK_COMBO(combo)->entry);
 		gtk_signal_connect(GTK_OBJECT(enqueue), "clicked", enqueue_func, GTK_COMBO(combo)->entry);
@@ -548,8 +605,6 @@ GtkWidget* util_create_add_url_window(gchar *caption, GtkSignalFunc ok_func, Gtk
 	gtk_box_pack_start(GTK_BOX(bbox), cancel, FALSE, FALSE, 0);
 	gtk_widget_show(cancel);
 	
-	gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
-	gtk_widget_show(bbox);
 	gtk_widget_show(vbox);
 	return win;
 }
@@ -564,19 +619,19 @@ static int int_compare_func(gconstpointer a, gconstpointer b)
 		return 0;
 }
 
-static void filebrowser_changed(GtkWidget * w, GtkFileSelection * filesel)
+static void filebrowser_changed(GtkFileSelection * filesel)
 {
-	gchar *current = "./", *parent = "../";
-	gchar *text = gtk_entry_get_text(GTK_ENTRY(w));
 	GList *list, *node;
+	char *filename = gtk_file_selection_get_filename(filesel);
 
-	if ((!text || !(*text)) &&
-	    (list = input_scan_dir(gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel)))) != NULL)
+	if ((list = input_scan_dir(filename)) != NULL)
 	{
 		/*
-		 * We enter a directory that has been "hijacked" by a
+		 * We enter a directory that has been "hijacked" by an
 		 * input-plugin. This is used by the CDDA plugin
 		 */
+		char *current = "./", *parent = "../";
+
 		gtk_clist_clear(GTK_CLIST(filesel->dir_list));
 		gtk_clist_append(GTK_CLIST(filesel->dir_list), &current);
 		gtk_clist_append(GTK_CLIST(filesel->dir_list), &parent);
@@ -586,7 +641,8 @@ static void filebrowser_changed(GtkWidget * w, GtkFileSelection * filesel)
 		node = list;
 		while (node)
 		{
-			gtk_clist_append(GTK_CLIST(filesel->file_list), (gchar **) & node->data);
+			gtk_clist_append(GTK_CLIST(filesel->file_list),
+					 (gchar **) & node->data);
 			g_free(node->data);
 			node = g_list_next(node);
 		}
@@ -595,32 +651,68 @@ static void filebrowser_changed(GtkWidget * w, GtkFileSelection * filesel)
 	}
 }
 
+static int filebrowser_idle_changed(gpointer data)
+{
+	GDK_THREADS_ENTER();
+	filebrowser_changed(GTK_FILE_SELECTION(data));
+	GDK_THREADS_LEAVE();
+
+	return FALSE;
+}
+
+static void filebrowser_entry_changed(GtkEditable *entry, gpointer data)
+{
+	char *text = gtk_entry_get_text(GTK_ENTRY(entry));
+
+	if (!text || !(*text))
+		filebrowser_changed(GTK_FILE_SELECTION(data));
+}
+
+ static void filebrowser_dir_select(GtkCList *clist, int row, int col, GdkEventButton *event, gpointer data)
+{
+	if (event->type == GDK_2BUTTON_PRESS)
+		gtk_idle_add(filebrowser_idle_changed, data);
+}
+
 gboolean util_filebrowser_is_dir(GtkFileSelection * filesel)
 {
-	gchar *text;
+	char *text;
 	struct stat buf;
+	gboolean retv = FALSE;
 	
 	text = g_strdup(gtk_file_selection_get_filename(filesel));
-	g_strstrip(text);
 
-	if (stat(text, &buf) == 0)
+	if (stat(text, &buf) == 0 &&
+	    S_ISDIR(buf.st_mode))
 	{
-		if (S_ISDIR(buf.st_mode))
-		{		/* Selected directory */
-			gtk_file_selection_set_filename(filesel, text);
-			g_free(text);
-			return TRUE;
+		/* Selected directory */
+		int len = strlen(text);
+		if (len > 3 && !strcmp(text + len - 4, "/../"))
+		{
+			if (len == 4)
+				/* At the root already */
+				*(text + len - 3) = '\0';
+			else
+			{
+				char *ptr;
+				*(text + len - 4) = '\0';
+				ptr = strrchr(text, '/');
+				*(ptr + 1) = '\0';
+			}
 		}
+		else if (len > 2 && !strcmp(text + len - 3, "/./"))
+			*(text + len - 2) = '\0';
+		gtk_file_selection_set_filename(filesel, text);
+		retv = TRUE;
 	}
 	g_free(text);
-	return FALSE;
+	return retv;
 }
 
 static void filebrowser_add_files(GtkFileSelection * filesel)
 {
 	GList *sel_list = NULL, *node;
-	gchar *text, *text2, *ptr;
-	gint *clear_on_ok;
+	char *text, *ptr;
 
 	if (cfg.filesel_path)
 		g_free(cfg.filesel_path);
@@ -640,37 +732,18 @@ static void filebrowser_add_files(GtkFileSelection * filesel)
 	}
 	sel_list = g_list_sort(sel_list, int_compare_func);
 
-	if (cfg.open_rev_order)
-		node = g_list_last(sel_list);
-	else
-		node = sel_list;
-
-	clear_on_ok = gtk_object_get_data(GTK_OBJECT(filesel),
-					  "clear_pl_on_ok");
-	if (clear_on_ok && *clear_on_ok == 2)
-	{
-		/*
-		 * clear the playlist if we're called
-		 * from the eject button
-		 */
-		playlist_clear();
-		*clear_on_ok = 1;
-	}
-
+	node = sel_list;
 	
 	if (node)
 	{
-		while (node)
-		{
-			gtk_clist_get_text(GTK_CLIST(filesel->file_list), GPOINTER_TO_INT(node->data), 0, &text);
-			text2 = g_strconcat(cfg.filesel_path, text, NULL);
-			playlist_add(text2);
-			g_free(text2);
-			if(cfg.open_rev_order)
-				node = g_list_previous(node);
-			else
-				node = g_list_next(node);
-		}
+		do {
+			char *tmp;
+			gtk_clist_get_text(GTK_CLIST(filesel->file_list),
+					   GPOINTER_TO_INT(node->data), 0, &text);
+			tmp = g_strconcat(cfg.filesel_path, text, NULL);
+			playlist_add(tmp);
+			g_free(tmp);
+		} while ((node = g_list_next(node)) != NULL);
 	}
 	else
 	{
@@ -681,25 +754,32 @@ static void filebrowser_add_files(GtkFileSelection * filesel)
 		text = gtk_file_selection_get_filename(filesel);
 		if (text[strlen(text) - 1] != '/')
 			playlist_add(text);
-		gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), "");
+		gtk_file_selection_set_filename(filesel, "");
 	}
 	g_list_free(sel_list);
 	playlistwin_update_list();
 }
 
-static void filebrowser_ok(GtkWidget * w, GtkWidget * filesel)
+static void filebrowser_add(GtkWidget * w, GtkWidget * filesel)
 {
-	gboolean *clear_on_ok = gtk_object_get_data(GTK_OBJECT(filesel),
-						    "clear_pl_on_ok");
 	if (util_filebrowser_is_dir(GTK_FILE_SELECTION(filesel)))
 		return;
 	gtk_widget_hide(filesel);
 	filebrowser_add_files(GTK_FILE_SELECTION(filesel));
-	if (clear_on_ok && *clear_on_ok > 0)
-		playlist_play();
 	gtk_widget_destroy(filesel);
 }
 
+static void filebrowser_play(GtkWidget * w, GtkWidget * filesel)
+{
+	if (util_filebrowser_is_dir(GTK_FILE_SELECTION(filesel)))
+		return;
+	playlist_clear();
+	gtk_widget_hide(filesel);
+	filebrowser_add_files(GTK_FILE_SELECTION(filesel));
+	gtk_widget_destroy(filesel);
+	playlist_play();
+}
+
 static void filebrowser_add_selected_files(GtkWidget * w, gpointer data)
 {
 	GtkFileSelection *filesel = GTK_FILE_SELECTION(data);
@@ -729,62 +809,104 @@ static void filebrowser_add_all_files(GtkWidget * w, gpointer data)
 	gtk_entry_set_text(GTK_ENTRY(filesel->selection_entry), "");
 }
 
-GtkWidget * util_create_filebrowser(gboolean clear_pl_on_ok)
+GtkWidget * util_create_filebrowser(gboolean play_button)
 {
-	GtkWidget *filebrowser, *bbox, *add_selected, *add_all, *label;
-	gboolean *ptr;
+	GtkWidget *filebrowser, *bbox, *add_selected, *add_all, *label, *button;
+	GtkFileSelection *fb;
+	GtkSignalFunc sf;
+	char *title;
 
-	filebrowser = gtk_file_selection_new(_("Load file(s)"));
+	if (play_button)
+		title = _("Play files");
+	else
+		title = _("Load files");
+
+	filebrowser = gtk_file_selection_new(title);
+	fb = GTK_FILE_SELECTION(filebrowser);
 		
-	gtk_clist_set_selection_mode(GTK_CLIST(GTK_FILE_SELECTION(filebrowser)->file_list),
+	gtk_clist_set_selection_mode(GTK_CLIST(fb->file_list),
 				     GTK_SELECTION_EXTENDED);
-	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filebrowser)->selection_entry),
-			   "changed", GTK_SIGNAL_FUNC(filebrowser_changed), filebrowser);
-	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filebrowser)->ok_button),
-			   "clicked", GTK_SIGNAL_FUNC(filebrowser_ok), filebrowser);
-	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filebrowser)->cancel_button),
-				  "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
+	gtk_signal_connect(GTK_OBJECT(fb->selection_entry), "changed",
+			   filebrowser_entry_changed, filebrowser);
+	gtk_signal_connect(GTK_OBJECT(fb->dir_list), "select_row",
+			   filebrowser_dir_select, filebrowser);
+	if (play_button)
+		sf = filebrowser_play;
+	else
+		sf = filebrowser_add;
+	gtk_signal_connect(GTK_OBJECT(fb->ok_button),
+			   "clicked", sf, filebrowser);
+	gtk_signal_connect_object(GTK_OBJECT(fb->cancel_button), "clicked",
+				  GTK_SIGNAL_FUNC(gtk_widget_destroy),
 				  GTK_OBJECT(filebrowser));
 
 	if (cfg.filesel_path)
-		gtk_file_selection_set_filename(GTK_FILE_SELECTION(filebrowser),
-						cfg.filesel_path);
+		gtk_file_selection_set_filename(fb, cfg.filesel_path);
 	bbox = gtk_hbutton_box_new();
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
 	gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 0);
-	gtk_box_pack_end(GTK_BOX(GTK_FILE_SELECTION(filebrowser)->action_area),
-			 bbox, TRUE, TRUE, 0);
+	gtk_box_pack_end(GTK_BOX(fb->action_area), bbox, TRUE, TRUE, 0);
 	add_selected  = gtk_button_new_with_label(_("Add selected files"));
 	gtk_box_pack_start(GTK_BOX(bbox), add_selected, FALSE, FALSE, 0);
 	gtk_signal_connect(GTK_OBJECT(add_selected), "clicked",
-			   GTK_SIGNAL_FUNC(filebrowser_add_selected_files), filebrowser);
+			   filebrowser_add_selected_files, filebrowser);
 	add_all = gtk_button_new_with_label(_("Add all files in directory"));
 	gtk_box_pack_start(GTK_BOX(bbox), add_all, FALSE, FALSE, 0);
 	gtk_signal_connect(GTK_OBJECT(add_all), "clicked",
-			   GTK_SIGNAL_FUNC(filebrowser_add_all_files), filebrowser);
+			   filebrowser_add_all_files, filebrowser);
 	gtk_widget_show_all(bbox);
 
 	/*
-	 * This is ugly.
 	 * Change the Cancel buttons caption to Close.
 	 */
 	
 	label = gtk_label_new(_("Close"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
-	gtk_container_remove(GTK_CONTAINER(GTK_FILE_SELECTION(filebrowser)->cancel_button),
-			     gtk_container_children(GTK_CONTAINER(GTK_FILE_SELECTION(filebrowser)->cancel_button))->data);
-	gtk_container_add(GTK_CONTAINER(GTK_FILE_SELECTION(filebrowser)->cancel_button), label);
+	gtk_container_remove(GTK_CONTAINER(fb->cancel_button),
+			     GTK_BIN(fb->cancel_button)->child);
+	gtk_container_add(GTK_CONTAINER(fb->cancel_button), label);
 	gtk_widget_show(label);
 
-	ptr = g_malloc(sizeof (gint));
-	*ptr = !!clear_pl_on_ok * 2;
-	gtk_object_set_data_full(GTK_OBJECT(filebrowser), "clear_pl_on_ok", ptr, g_free_func);
+	/*
+	 * Change the OK buttons caption to Add or Play.
+	 */
+
+	if (play_button)
+		label = gtk_label_new(_("Play"));
+	else
+		label = gtk_label_new(_("Add"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
+	gtk_container_remove(GTK_CONTAINER(fb->ok_button),
+			     GTK_BIN(fb->ok_button)->child);
+	gtk_container_add(GTK_CONTAINER(fb->ok_button), label);
+	gtk_widget_show(label);
+
+	if (play_button)
+	{
+		GtkArg arg;
+
+		/*
+		 * The amount of fiddling we do with the filesel
+		 * internals are starting to get ridicolous.  Maybe we
+		 * should copy the entire filesel code instead.
+		 */
+		arg.name = "GtkWidget::parent";
+		gtk_object_arg_get(GTK_OBJECT(fb->ok_button), &arg, NULL);
+		button = gtk_button_new_with_label("Add");
+		gtk_signal_connect(GTK_OBJECT(button), "clicked",
+				   filebrowser_add, filebrowser);
+		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+		gtk_box_pack_start(GTK_BOX(arg.d.object_data),
+				   button, TRUE, TRUE, 0);
+		gtk_box_reorder_child(GTK_BOX(arg.d.object_data), button, 1);
+		gtk_widget_show(button);
+	}
 
 	gtk_widget_show(filebrowser);
 	return filebrowser;
 }
 
-GdkFont *util_font_load(gchar *name)
+GdkFont *util_font_load(char *name)
 {
 	GdkFont *font;
 
@@ -800,10 +922,18 @@ GdkFont *util_font_load(gchar *name)
 		if ((font = gdk_fontset_load(name)) == NULL)
 			font = gdk_font_load(name);
 	}
+
+	if (!font)
+	{
+		g_warning("Failed to open font: \"%s\".", name);
+		font = gdk_font_load("fixed");
+	}
+	
 	return font;
 }
 
-gchar* util_menu_translate(const gchar *path, gpointer func_data)
+#ifdef ENABLE_NLS
+char* util_menu_translate(const char *path, void *func_data)
 {
 	char *translation = gettext(path);
 
@@ -815,6 +945,40 @@ gchar* util_menu_translate(const gchar *path, gpointer func_data)
 
 	return translation;
 }
+#endif
+
+void util_set_cursor(GtkWidget *window)
+{
+	static GdkCursor *cursor;
+
+	if (!window)
+	{
+		if (cursor)
+		{
+			gdk_cursor_destroy(cursor);
+			cursor = NULL;
+		}
+		return;
+	}
+	if (!cursor)
+		cursor = gdk_cursor_new(GDK_LEFT_PTR);
+
+	gdk_window_set_cursor(window->window, cursor);
+}
+
+void util_dump_menu_rc(void)
+{
+	char *filename = g_strconcat(g_get_home_dir(), "/.xmms/menurc", NULL);
+	gtk_item_factory_dump_rc(filename, NULL, FALSE);
+	g_free(filename);
+}
+
+void util_read_menu_rc(void)
+{
+	char *filename = g_strconcat(g_get_home_dir(), "/.xmms/menurc", NULL);
+	gtk_item_factory_parse_rc(filename);
+	g_free(filename);
+}
 
 
 #if defined(USE_DMALLOC)
